# http://tagmask.com/vladimir/posts/26
#Copyright (c) 2011 Vladimir Pankratiev http://tagmask.com 
#
#Permission is hereby granted, free of charge, to any person obtaining a copy
#of this software and associated documentation files (the "Software"), to deal
#in the Software without restriction, including without limitation the rights
#to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#copies of the Software, and to permit persons to whom the Software is
#furnished to do so, subject to the following conditions:
#
#The above copyright notice and this permission notice shall be included in
#all copies or substantial portions of the Software.
#
#THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
#THE SOFTWARE.

import httplib
import urllib
import hashlib
import hmac
import logging
import base64
from datetime import datetime
from xml.etree.ElementTree import XML
from django.conf import settings

log = logging.getLogger(__name__)

class AmazonSES:
    def __init__(self):
        self._accessKeyID = getattr(settings, 'AWS_ACCESS_KEY_ID', None)
        self._secretAccessKey = getattr(settings, 'AWS_SECRET_ACCESS_KEY', None)
        self._responseParser = AmazonResponseParser()

    def _getSignature(self, dateValue):
        h = hmac.new(key=self._secretAccessKey, msg=dateValue, digestmod=hashlib.sha256)
        return base64.b64encode(h.digest()).decode()
        
    def _getHeaders(self):
        headers = { 'Content-type': 'application/x-www-form-urlencoded' }
        d = datetime.utcnow()
        dateValue = d.strftime('%a, %d %b %Y %H:%M:%S GMT')
        headers['Date'] = dateValue
        signature = self._getSignature(dateValue)
        headers['X-Amzn-Authorization'] = 'AWS3-HTTPS AWSAccessKeyId=%s, Algorithm=HMACSHA256, Signature=%s' % (self._accessKeyID, signature)
        return headers
    
    def _performAction(self, actionName, params=None):
        if not params:
            params = {}
        params['Action'] = actionName        
        #https://email.us-east-1.amazonaws.com/
        conn = httplib.HTTPSConnection('email.us-east-1.amazonaws.com')
        params = urllib.urlencode(params)
        conn.request('POST', '/', params, self._getHeaders())
        response = conn.getresponse()
        responseResult = response.read()
        conn.close()
        return self._responseParser.parse(actionName, response.status, response.reason, responseResult)
        
    def verifyEmailAddress(self, emailAddress):
        params = { 'EmailAddress': emailAddress }
        return self._performAction('VerifyEmailAddress', params)
        
    def deleteVerifiedEmailAddress(self, emailAddress):
        params = { 'EmailAddress': emailAddress }
        return self._performAction('DeleteVerifiedEmailAddress', params)
        
    def getSendQuota(self):
        return self._performAction('GetSendQuota')
        
    def getSendStatistics(self):
        return self._performAction('GetSendStatistics')
        
    def listVerifiedEmailAddresses(self):
        return self._performAction('ListVerifiedEmailAddresses')
        
    def sendEmail(self, source, toAddresses, message, replyToAddresses=None, returnPath=None, ccAddresses=None, bccAddresses=None):
        params = { 'Source': source }
        for objName, addresses in zip(["ToAddresses", "CcAddresses", "BccAddresses"], [toAddresses, ccAddresses, bccAddresses]):
            if addresses:
                if not isinstance(addresses, basestring) and getattr(addresses, '__iter__', False):
                    for i, address in enumerate(addresses, 1):
                        params['Destination.%s.member.%d' % (objName, i)] = address
                else:
                    params['Destination.%s.member.1' % objName] = addresses                    
        if not returnPath:
            returnPath = source
        params['ReturnPath'] = returnPath        
        params['Message.Subject.Charset'] = message.charset
        params['Message.Subject.Data'] = message.subject
        if message.bodyText:
            params['Message.Body.Text.Charset'] = message.charset
            params['Message.Body.Text.Data'] = message.bodyText
        if message.bodyHtml:
            params['Message.Body.Html.Charset'] = message.charset
            params['Message.Body.Html.Data'] = message.bodyHtml
        return self._performAction('SendEmail', params)



class EmailMessage:
    def __init__(self):
        self.charset = 'UTF-8'
        self.subject = None
        self.bodyHtml = None
        self.bodyText = None    



class AmazonError(Exception):
    def __init__(self, errorType, code, message):
        self.errorType = errorType
        self.code = code
        self.message = message
    
class AmazonAPIError(Exception):
    def __init__(self, message):
        self.message = message
    
    
    
class AmazonResult:
    def __init__(self, requestId):
        self.requestId = requestId
        
class AmazonSendEmailResult(AmazonResult):
    def __init__(self, requestId, messageId):
        self.requestId = requestId
        self.messageId = messageId
        
class AmazonSendQuota(AmazonResult):
    def __init__(self, requestId, max24HourSend, maxSendRate, sentLast24Hours):
        self.requestId = requestId
        self.max24HourSend = max24HourSend
        self.maxSendRate = maxSendRate
        self.sentLast24Hours = sentLast24Hours
        
class AmazonSendDataPoint:
    def __init__(self, bounces, complaints, deliveryAttempts, rejects, timestamp):
        self.bounces = bounces
        self.complaints = complaints
        self.deliveryAttempts = deliveryAttempts
        self.rejects = rejects
        self.timestamp = timestamp
    
class AmazonSendStatistics(AmazonResult):
    def __init__(self, requestId):
        self.requestId = requestId
        self.members = []        
        
class AmazonVerifiedEmails(AmazonSendStatistics):
    pass
        
class AmazonResponseParser:
    class XmlResponse:
        def __init__(self, str):
            self._rootElement = XML(str)
            self._namespace = self._rootElement.tag[1:].split("}")[0]
            
        def checkResponseName(self, name):
            if self._rootElement.tag == self._fixTag(self._namespace, name):
                return True
            else:
                raise AmazonAPIError('ErrorResponse is invalid.')            
        
        def checkActionName(self, actionName):
            if self._rootElement.tag == self._fixTag(self._namespace, ('%sResponse' % actionName)):
                return True
            else:
                raise AmazonAPIError('Response of action "%s" is invalid.' % actionName)
        
        def getChild(self, *itemPath):
            node = self._findNode(self._rootElement, self._namespace, *itemPath)
            if node != None:
                return node
            else:
                raise AmazonAPIError('Node with the specified path was not found.')
        
        def getChildText(self, *itemPath):            
            node = self.getChild(*itemPath)            
            return node.text
        
        def _fixTag(self, namespace, tag):
            return '{%s}%s' % (namespace, tag)

        def _findNode(self, rootElement, namespace, *args):
            match = '.'
            for s in args:
                match += '/{%s}%s' % (namespace, s)            
            return rootElement.find(match)            
   
    
    def __init__(self):
        self._simpleResultActions = ['DeleteVerifiedEmailAddress', 'VerifyEmailAddress']
        
    def _parseSimpleResult(self, actionName, xmlResponse):        
        if xmlResponse.checkActionName(actionName):
            requestId = xmlResponse.getChildText('ResponseMetadata', 'RequestId')
            return AmazonResult(requestId)
                
    def _parseSendQuota(self, actionName, xmlResponse):
        if xmlResponse.checkActionName(actionName):
            requestId = xmlResponse.getChildText('ResponseMetadata', 'RequestId')
            value = xmlResponse.getChildText('GetSendQuotaResult', 'Max24HourSend')            
            max24HourSend = float(value)
            value = xmlResponse.getChildText('GetSendQuotaResult', 'MaxSendRate')
            maxSendRate = float(value)
            value = xmlResponse.getChildText('GetSendQuotaResult', 'SentLast24Hours')
            sentLast24Hours = float(value)
            return AmazonSendQuota(requestId, max24HourSend, maxSendRate, sentLast24Hours)
            
    #def _parseSendStatistics(self, actionName, xmlResponse):
    #    if xmlResponse.checkActionName(actionName):
    #        requestId = xmlResponse.getChildText('ResponseMetadata', 'RequestId')
    
    def _parseListVerifiedEmails(self, actionName, xmlResponse):
        if xmlResponse.checkActionName(actionName):
            requestId = xmlResponse.getChildText('ResponseMetadata', 'RequestId')
            node = xmlResponse.getChild('ListVerifiedEmailAddressesResult', 'VerifiedEmailAddresses')
            result = AmazonVerifiedEmails(requestId)
            for addr in node:                
                result.members.append(addr.text)
            return result
                
    def _parseSendEmail(self, actionName, xmlResponse):
        if xmlResponse.checkActionName(actionName):
            requestId = xmlResponse.getChildText('ResponseMetadata', 'RequestId')
            messageId = xmlResponse.getChildText('SendEmailResult', 'MessageId')
            return AmazonSendEmailResult(requestId, messageId)
    
    def _raiseError(self, xmlResponse):
        if xmlResponse.checkResponseName('ErrorResponse'):
            errorType = xmlResponse.getChildText('Error', 'Type')
            code = xmlResponse.getChildText('Error', 'Code')
            message = xmlResponse.getChildText('Error', 'Message')
            raise AmazonError(errorType, code, message)
            
    def parse(self, actionName, statusCode, reason, responseResult):        
        xmlResponse = self.XmlResponse(responseResult)
        log.info('Response status code: %s, reason: %s', statusCode, reason)
        log.debug(responseResult)
        
        result = None                
        if statusCode != 200:
            self._raiseError(xmlResponse)
        else:
            if actionName in self._simpleResultActions:
                result = self._parseSimpleResult(actionName, xmlResponse)
            elif actionName in ['SendEmail']:
                result = self._parseSendEmail(actionName, xmlResponse)
            elif actionName == 'GetSendQuota':
                result = self._parseSendQuota(actionName, xmlResponse)
            #elif actionName == 'GetSendStatistics':
            #    result = self._parseSendStatistics(actionName, xmlResponse)
            elif actionName == 'ListVerifiedEmailAddresses':
                result = self._parseListVerifiedEmails(actionName, xmlResponse)                
            else:
                #raise AmazonAPIError('Action %s is not supported. Please contact: vladimir@tagmask.com' % (actionName,))
                raise AmazonAPIError('Action %s is not supported.' % (actionName,))
        return result



from haystack import indexes
from haystack import site

from django.utils.html import strip_tags, strip_entities

from tendenci.addons.jobs.models import Job
from tendenci.core.perms.object_perms import ObjectPermission
from tendenci.core.categories.models import Category
from tendenci.core.perms.indexes import TendenciBaseSearchIndex

class JobIndex(TendenciBaseSearchIndex):
    title = indexes.CharField(model_attr='title')
    list_type = indexes.CharField(model_attr='list_type')
    description = indexes.CharField(model_attr='description')
    post_dt = indexes.DateTimeField(model_attr='post_dt', null=True)
    syndicate = indexes.BooleanField(model_attr='syndicate')
    
    # categories
    category = indexes.CharField()
    sub_category = indexes.CharField()
    
    # RSS fields
    can_syndicate = indexes.BooleanField()
    
    # PK: needed for exclude list_tags
    primary_key = indexes.CharField(model_attr='pk')
    
    #overriden fields
    creator = indexes.CharField(model_attr='creator', null=True)
    creator_username = indexes.CharField(model_attr='creator_username', null=True)
    owner = indexes.CharField(model_attr='owner', null=True)
    owner_username = indexes.CharField(model_attr='owner_username', null=True)
    
    def prepare_description(self, obj):
        description = obj.description
        description = strip_tags(description)
        description = strip_entities(description)
        return description

    def prepare_category(self, obj):
        category = Category.objects.get_for_object(obj, 'category')
        if category:
            return category.name
        return ''

    def prepare_sub_category(self, obj):
        category = Category.objects.get_for_object(obj, 'sub_category')
        if category:
            return category.name
        return ''

    def prepare_can_syndicate(self, obj):
        return obj.allow_anonymous_view and obj.syndicate \
                and obj.status == 1 and obj.status_detail == 'active'

site.register(Job, JobIndex)



# models.py
from dateutil.parser import parse

from django.db import models
from django.contrib.auth.models import User
from django.contrib.contenttypes.models import ContentType
from django.utils.translation import ugettext_lazy as _
from django.contrib.contenttypes import generic
from django.utils import simplejson as json

from tendenci.core.versions.managers import VersionManager


class Version(models.Model):
    """
    Creates a historical version of an object.
    Stores the creator, create_dt, and the object serialized in json
    """

    create_dt = models.DateTimeField(_('create time'))
    user = models.ForeignKey(User, null=True, on_delete=models.SET_NULL)
    content_type = models.ForeignKey(ContentType)
    object_id = models.IntegerField(_('object id'))
    object_repr = models.CharField(_('object repr'), max_length=200)
    object_changes = models.TextField(_('change message'), blank=True)
    object_value = models.TextField(_('changed object'), blank=True)
    hash = models.CharField(max_length=40, null=True, default='')

    _object = generic.GenericForeignKey('content_type', 'object_id')

    objects = VersionManager()

    def get_object(self):
        _object = None
        try:
            _object = self._object
        except:
            pass
        return _object

    def get_version_object(self):
        data = json.loads(self.object_value)['fields']
        obj_data = {'pk': self.object_id}
        for f in self.get_object()._meta.fields:
            field_name = unicode(f.name)
            if field_name in data:
                #print unicode(f.get_internal_type())
                if unicode(f.get_internal_type()) == 'ForeignKey' or unicode(f.get_internal_type()) == 'OneToOneField':
                    obj_data[field_name + "_id"] = data[field_name]
                elif unicode(f.get_internal_type()) == 'DateTimeField':
                    obj_data[field_name] = parse(data[field_name])
                else:
                    obj_data[field_name] = data[field_name]
        obj = self.get_object().__class__(**obj_data)

        return obj

    def get_object_version_url(self):
        try:
            return self.get_version_object().get_version_url(self.hash)
        except:
            return ''



import operator
from uuid import uuid4
from os.path import join
from datetime import datetime
from django import forms
from django.utils.translation import ugettext_lazy as _
from django.core.files.storage import FileSystemStorage
from django.forms.fields import ChoiceField
#from django.template.defaultfilters import slugify
from django.contrib.auth.models import User
from django.db.models import Q

from captcha.fields import CaptchaField
from tinymce.widgets import TinyMCE

from tendenci.addons.memberships.fields import PriceInput
from tendenci.addons.corporate_memberships.models import (
                    CorporateMembershipType,
                    CorpApp,
                    CorpField,
                    CorporateMembership,
                    Creator,
                    CorporateMembershipRep,
                    CorpMembRenewEntry)
from tendenci.addons.corporate_memberships.utils import (
                 get_corpapp_default_fields_list,
                 update_auth_domains,
                 get_payment_method_choices,
                 get_indiv_membs_choices,
                 get_corporate_membership_type_choices,
                 csv_to_dict)
from tendenci.addons.corporate_memberships.settings import UPLOAD_ROOT
from tendenci.core.base.fields import SplitDateTimeField
from tendenci.core.payments.models import PaymentMethod

fs = FileSystemStorage(location=UPLOAD_ROOT)


class CorporateMembershipTypeForm(forms.ModelForm):
    description = forms.CharField(label=_('Description'),
                                  max_length=500,
                                  required=False,
                               widget=forms.Textarea(
                                    attrs={'rows': '3'}))
    price = forms.DecimalField(decimal_places=2, widget=PriceInput(),
                               help_text="Set 0 for free membership.")
    renewal_price = forms.DecimalField(decimal_places=2,
                                       widget=PriceInput(),
                                       required=False,
                               help_text="Set 0 for free membership.")
    status_detail = forms.ChoiceField(
        choices=(('active', 'Active'),
                 ('inactive', 'Inactive'),
                 ('admin hold', 'Admin Hold'),))

    class Meta:
        model = CorporateMembershipType
        fields = (
                  'name',
                  'price',
                  'renewal_price',
                  'membership_type',
                  'description',
                  'apply_threshold',
                  'individual_threshold',
                  'individual_threshold_price',
                  'admin_only',
                  'order',
                  'status',
                  'status_detail',
                  )


class CorpAppForm(forms.ModelForm):
    description = forms.CharField(required=False,
                     widget=TinyMCE(
                    attrs={'style': 'width:70%'},
                    mce_attrs={
                   'storme_app_label': CorpApp._meta.app_label,
                   'storme_model': CorpApp._meta.module_name.lower()}),
                   help_text='Will show at the top of the application form.')
    confirmation_text = forms.CharField(required=False,
                 widget=TinyMCE(
                    attrs={'style': 'width:70%'},
                    mce_attrs={'storme_app_label': CorpApp._meta.app_label,
                               'storme_model': CorpApp._meta.module_name.lower()}),
                               help_text='Will show on the confirmation page.')
    notes = forms.CharField(label=_('Notes'), required=False,
               widget=forms.Textarea(attrs={'rows': '3'}),
               help_text='Notes for editor. Will not display on the application form.')
    status_detail = forms.ChoiceField(
        choices=(('active', 'Active'),
                 ('inactive', 'Inactive'),
                 ('admin hold', 'Admin Hold'),))

    class Meta:
        model = CorpApp
        fields = (
                  'name',
                  'slug',
                  'corp_memb_type',
                  'authentication_method',
                  'memb_app',
                  'payment_methods',
                  'description',
                  'confirmation_text',
                  'notes',
                  #'use_captcha',
                  #'require_login',
                  'status',
                  'status_detail',
                  )

    def __init__(self, *args, **kwargs): 
        super(CorpAppForm, self).__init__(*args, **kwargs)
        if self.instance.pk:
            self.fields['description'].widget.mce_attrs['app_instance_id'] = self.instance.pk
            self.fields['confirmation_text'].widget.mce_attrs['app_instance_id'] = self.instance.pk
        else:
            self.fields['description'].widget.mce_attrs['app_instance_id'] = 0
            self.fields['confirmation_text'].widget.mce_attrs['app_instance_id'] = 0

default_corpapp_inline_fields_list = get_corpapp_default_fields_list()
if default_corpapp_inline_fields_list:
    required_corpapp_inline_fields_list = [
                    str(field_d['field_name']) for field_d in \
                    default_corpapp_inline_fields_list  \
                    if field_d['required']]
else:
    required_corpapp_inline_fields_list = []


class CorpFieldForm(forms.ModelForm):
    instruction = forms.CharField(label=_('Instruction for User'),
                          max_length=500,
                          required=False,
                          widget=forms.Textarea(
                                attrs={'rows': '3'}))
    field_name = forms.CharField(label=(''), max_length=30,
                                 required=False,
                               widget=forms.HiddenInput())

    class Meta:
        model = CorpField
        fields = (
                  'label',
                  'field_name',
                  #'object_type',
                  'field_type',
                  'size',
                  'choices',
                  'field_layout',
                  'required',
                  'visible',
                  'no_duplicates',
                  'instruction',
                  'default_value',
                  'admin_only',
                  'css_class',
                  'order'
                  )

    def __init__(self, *args, **kwargs):
        super(CorpFieldForm, self).__init__(*args, **kwargs)

        instance = getattr(self, 'instance', None)
        if instance and instance.id:
            if instance.field_name in required_corpapp_inline_fields_list \
            and instance.required:
                self.fields['required'].widget.attrs['disabled'] = "disabled"
                self.fields['visible'].widget.attrs['disabled'] = "disabled"
            if instance.field_name == 'name':
                self.fields['no_duplicates'].widget.attrs['disabled'] = "disabled"

    def clean_required(self):
        if self.instance.field_name in required_corpapp_inline_fields_list \
         and self.instance.required:
            return self.instance.required
        return self.cleaned_data['required']

    def clean_visible(self):
        if self.instance.field_name in required_corpapp_inline_fields_list \
         and self.instance.visible:
            return self.instance.visible
        return self.cleaned_data['visible']

    def clean_no_duplicates(self):
        if self.instance.field_name == 'name' and self.instance.no_duplicates:
            return self.instance.no_duplicates
        return self.cleaned_data['no_duplicates']


class CorpMembForm(forms.ModelForm):
    status_detail = forms.ChoiceField(
        choices=(('active', 'Active'),
                 ('pending', 'Pending'),
                 ('paid - pending approval', 'Paid - Pending Approval'),
                 ('admin hold', 'Admin Hold'),
                 ('inactive', 'Inactive'),
                 ('expired', 'Expired'),))
    join_dt = SplitDateTimeField(label=_('Join Date/Time'),
        initial=datetime.now())
    expiration_dt = SplitDateTimeField(
                       label=_('Expiration Date/Time'),
                       required=False,
                       help_text='Not specified = Never expires')

    class Meta:
        model = CorporateMembership
        exclude = ('corp_app', 'guid', 'renewal', 'invoice',
                   'renew_dt', 'secret_code',
                   'approved', 'approved_denied_dt',
                   'approved_denied_user',
                   'creator_username',
                   'owner', 'owner_username',
                   'anonymous_creator')

    def __init__(self, corp_app, field_objs, *args, **kwargs):
        """
            Dynamically build the form fields.
        """
        self.corp_app = corp_app
        self.field_objs = field_objs
        super(CorpMembForm, self).__init__(*args, **kwargs)

        for field in field_objs:
            if field.field_type not in ['section_break', 'page_break']:
                if field.field_name:
                    field_key = field.field_name
                else:
                    field_key = "field_%s" % field.id

                # if field is display only, remove it from the form
                # for example, on the edit page, we
                # set corporate_membership_type
                # and payment_method as the display only fields
                if hasattr(field, 'display_only') and field.display_only:
                    del self.fields[field_key]
                else:
                    # get field class and set field initial
                    self.fields[field_key] = field.get_field_class()
                    if ((not field.field_name) \
                        or field.field_name == 'authorized_domains') \
                        and self.instance:
                        initial = field.get_value(self.instance)
                        if field.field_type in [
                            'MultipleChoiceField/django.forms.CheckboxSelectMultiple',
                            'MultipleChoiceField']:
                            if initial:
                                self.fields[field_key].initial = [
                                item.strip() for item in initial.split(',')]
                        else:
                            self.fields[field_key].initial = initial

        #self.fields['captcha'] = CaptchaField(label=_('Type the code below'))

    def clean_corporate_membership_type(self):
        if self.cleaned_data['corporate_membership_type']:
            return CorporateMembershipType.objects.get(
            pk=int(self.cleaned_data['corporate_membership_type']))
        return self.cleaned_data['corporate_membership_type']

    def clean_secret_code(self):
        secret_code = self.cleaned_data['secret_code']
        if secret_code:
            # check if this secret_code is available to ensure the uniqueness
            corp_membs = CorporateMembership.objects.filter(
                                secret_code=secret_code)
            if self.instance:
                corp_membs = corp_membs.exclude(id=self.instance.id)
            if corp_membs:
                raise forms.ValidationError(
            _("This secret code is already taken. Please use a different one.")
            )
        return self.cleaned_data['secret_code']

    def clean_payment_method(self):
        if self.cleaned_data['payment_method']:
            return PaymentMethod.objects.get(
                pk=int(self.cleaned_data['payment_method'])
                )
        return self.cleaned_data['payment_method']

    def save(self, user, **kwargs):
        """
            Create a CorporateMembership instance and related
            CorpFieldEntry instances for each form field.
        """
        corporate_membership = super(CorpMembForm, self).save(commit=False)
        corporate_membership.corp_app = self.corp_app
        creator_owner = user

        if not self.instance.pk:
            mode = 'add'
        else:
            mode = 'edit'

        if mode == 'add':
            anonymous_creator = kwargs.get('creator', None)
            if anonymous_creator:
                corporate_membership.anonymous_creator = anonymous_creator
            if not isinstance(creator_owner, User):
                # if anonymous is creating the corporate membership
                # temporarily use the first admin, the creator will be assigned 
                # back to the real user on approval
                tmp_user = User.objects.filter(is_staff=1, is_active=1)[0]
                creator_owner = tmp_user

            corporate_membership.creator = creator_owner
            corporate_membership.creator_username = creator_owner.username

            if not user.profile.is_superuser:
                corporate_membership.status = 1
                corporate_membership.status_detail = 'pending'
                corporate_membership.join_dt = datetime.now()

        corporate_membership.owner = creator_owner
        corporate_membership.owner_username = creator_owner.username

            # calculate the expiration dt
        corporate_membership.save()
        for field_obj in self.field_objs:
            if (not field_obj.field_name) and field_obj.field_type not in [
                'section_break', 'page_break']:
                field_key = "field_%s" % field_obj.id
                value = self.cleaned_data[field_key]
                if value and self.fields[field_key].widget.needs_multipart_form:
                    if not type(value) is unicode:
                        value = fs.save(join("forms",
                                        str(uuid4()),
                                        value.name),
                                        value)
                # if the value is a list convert is to a comma delimited string
                if isinstance(value, list):
                    value = ','.join(value)
                if not value:
                    value = ''

                if hasattr(field_obj, 'entry') and field_obj.entry:
                    field_obj.entry.value = value
                    field_obj.entry.save()
                else:
                    corporate_membership.fields.create(field_id=field_obj.id,
                                                       value=value)

        # update authorized domain if needed
        if self.corp_app.authentication_method == 'email':
            update_auth_domains(corporate_membership,
                                self.cleaned_data['authorized_domains'])

        return corporate_membership


class CreatorForm(forms.ModelForm):
    class Meta:
        model = Creator
        fields = ('first_name',
                'last_name',
                'email', )

    def __init__(self, *args, **kwargs):
        super(CreatorForm, self).__init__(*args, **kwargs)
        self.fields['captcha'] = CaptchaField(label=_('Type the code below'))


class CorpApproveForm(forms.Form):

    users = forms.ChoiceField(
        label='Assign creator/owner to this corporate membership',
        choices=[],
        widget=forms.RadioSelect,
        )

    def suggested_users(self, first_name='', last_name='', email=''):
        """
            Generate list of suggested users based on the given info.
            It queries (first name and last name) or (email) if provided.
        """
        user_set = {}

        qAnd = []
        query = None

        if email:
            query = Q(email=email)
        if first_name:
            qAnd.append(Q(first_name=first_name))
        if last_name:
            qAnd.append(Q(last_name=last_name))

        if qAnd:
            if query:
                query = reduce(operator.and_, qAnd) | query
            else:
                query = reduce(operator.and_, qAnd)

        if query:
            users = User.objects.filter(query).order_by('last_name')

            for u in users:
                user_set[u.pk] = '%s %s %s %s ' % (u.first_name,
                                                   u.last_name,
                                                   u.username,
                                                   u.email)

        return user_set.items()

    def __init__(self, *args, **kwargs):
        corp_memb = kwargs.pop('corporate_membership')
        super(CorpApproveForm, self).__init__(*args, **kwargs)

        if corp_memb.is_join_pending and corp_memb.anonymous_creator:
            suggested_users = self.suggested_users(
                           first_name=corp_memb.anonymous_creator.first_name,
                           last_name=corp_memb.anonymous_creator.last_name,
                           email=corp_memb.anonymous_creator.email)
            suggested_users.append((0, 'Create new user for %s %s %s' % (
                                  corp_memb.anonymous_creator.first_name,
                                  corp_memb.anonymous_creator.last_name,
                                  corp_memb.anonymous_creator.email
                                                         )))
            self.fields['users'].choices = suggested_users
            self.fields['users'].initial = 0
        else:
            self.fields.pop('users')


class CorpMembRepForm(forms.ModelForm):
    user_display = forms.CharField(max_length=100,
                        required=False,
                        help_text='type name, or username or email')

    class Meta:
        model = CorporateMembershipRep
        fields = ('user_display',
                 'user',
                'is_dues_rep',
                'is_member_rep',)
#        exclude = (
#                  'corporate_membership',
#                  )

    def __init__(self, corp_memb, *args, **kwargs):
        self.corporate_membership = corp_memb
        super(CorpMembRepForm, self).__init__(*args, **kwargs)

        self.fields['user_display'].label = "Add a Representative"
        self.fields['user'].widget = forms.HiddenInput()
        self.fields['user'].error_messages['required'
                                ] = 'Please enter a valid user.'

    def clean_user(self):
        value = self.cleaned_data['user']
        try:
            rep = CorporateMembershipRep.objects.get(
                corporate_membership=self.corporate_membership,
                user=value)
            raise forms.ValidationError(
                _("This user is already a representative."))
        except CorporateMembershipRep.DoesNotExist:
            pass
        return value


class RosterSearchForm(forms.Form):
    name = forms.CharField(max_length=100, widget=forms.HiddenInput())
    q = forms.CharField(max_length=100, required=False)


class CorpMembRenewForm(forms.ModelForm):
    members = forms.MultipleChoiceField(widget=forms.CheckboxSelectMultiple,
                                        choices=[],
                                        required=False)
    select_all_members = forms.BooleanField(widget=forms.CheckboxInput(),
                                            required=False)

    class Meta:
        model = CorpMembRenewEntry
        fields = ('corporate_membership_type',
                  'members',
                  'payment_method',
                  'select_all_members')

    def __init__(self, *args, **kwargs):
        user = kwargs.pop('user', None)
        corporate_membership = kwargs.pop('corporate_membership', None)

        super(CorpMembRenewForm, self).__init__(*args, **kwargs)

        self.fields['corporate_membership_type'].widget = forms.RadioSelect(
                    choices=get_corporate_membership_type_choices(user,
                    corporate_membership.corp_app,
                    renew=True))
        self.fields['corporate_membership_type'].empty_label = None
        self.fields['corporate_membership_type'
                ].initial = corporate_membership.corporate_membership_type.id

        members_choices = get_indiv_membs_choices(corporate_membership)
        self.fields['members'].choices = members_choices
        self.fields['members'].label = "Select the individual members you " + \
                                        "want to renew"
        if corporate_membership.corporate_membership_type.renewal_price == 0:
            self.fields['select_all_members'].initial = True 
            self.fields['members'].initial = [c[0] for c in members_choices]

        self.fields['payment_method'].widget = forms.RadioSelect(
                                    choices=get_payment_method_choices(
                                    user, corporate_membership.corp_app))
        self.fields['payment_method'].empty_label = None
        self.fields['payment_method'].initial = \
                corporate_membership.payment_method


class CSVForm(forms.Form):
    """
    Map CSV import to Membership Application.
    Create Membership Entry on save() method.
    """
    def __init__(self, *args, **kwargs):
        """
        Dynamically create fields using the membership
        application chosen.  The choices provided to these
        dynamic fields are the csv import columns.
        """
        step_numeral, step_name = kwargs.pop('step', (None, None))
        corp_app = kwargs.pop('corp_app', '')
        file_path = kwargs.pop('file_path', '')

        super(CSVForm, self).__init__(*args, **kwargs)

        if step_numeral == 1:
            """
            Basic Form: Application & File Uploader
            """
            self.fields['corp_app'] = forms.ModelChoiceField(
                label=_('Corp Application'), queryset=CorpApp.objects.all())

            self.fields['update_option'] = forms.CharField(
                    widget=forms.RadioSelect(
                        choices=(('skip', 'Skip'),
                                 ('update', 'Update Blank Fields'),
                                ('override', 'Override All Fields'),)),
                        initial='skip',
                        label=_('Select an Option for the Existing Records:')
                    )

            self.fields['csv'] = forms.FileField(label=_("CSV File"))

        if step_numeral == 2:
            """
            Basic Form + Mapping Fields
            """

            # file to make field-mapping form
            csv = csv_to_dict(file_path)

            # choices list
            choices = csv[0].keys()

            # make tuples; sort tuples (case-insensitive)
            choice_tuples = [(c, c) for c in csv[0].keys()]

            # insert blank option
            choice_tuples.insert(0, ('', ''))
            choice_tuples = sorted(choice_tuples, key=lambda c: c[0].lower())

            app_fields = CorpField.objects.filter(corp_app=corp_app)
            required_fields = ['name', 'corporate_membership_type']
            for field in app_fields:
                if field.field_type not in ['section_break', 'page_break']:
                    if field.field_name:
                        field_key = field.field_name
                    else:
                        field_key = "field_%s" % field.id
                    is_required = False
                    if field_key in required_fields:
                        is_required = True
                    self.fields[field_key] = ChoiceField(**{
                                                'label': field.label,
                                                'choices': choice_tuples,
                                                'required': is_required,
                                                })
                    for choice in choices:
                        if (field.label).lower() == choice.lower() or \
                            field_key.lower() == choice.lower():
                            self.fields[field_key].initial = choice

            extra_fields = (('secret_code', 'Secret Code'),
                            ('join_dt', 'Join Date'),
                            ('renew_dt', 'Renew Date'),
                            ('expiration_dt', 'Expiration Date'),
                            ('approved', 'Approved'),
                            ('dues_rep', 'Dues Representative'),
                            ('status', 'Status'),
                            ('status_detail', 'Status Detail'))
            # corp_memb_field_names = [smart_str(field.name)
            # for field in CorporateMembership._meta.fields]
            for key, label in extra_fields:
                if key not in self.fields.keys():
                    self.fields[key] = ChoiceField(**{
                                            'label': label,
                                            'choices': choice_tuples,
                                            'required': False,
                                            })
                    for choice in choices:
                        if label.lower() == choice.lower() or \
                         key.lower() == choice.lower():
                            self.fields[key].initial = choice

    def clean_csv(self):
        csv = self.cleaned_data['csv']
        SUPPORTED_FILE_TYPES = ['text/csv',]

        if not csv.content_type in SUPPORTED_FILE_TYPES:
            raise forms.ValidationError(_('File type is not supported. Please upload a CSV File.'))
        return csv

    def save(self, *args, **kwargs):
        """
        Loop through the dynamic fields and create a
        corporate membership record.
        """
        step_numeral, step_name = kwargs.pop('step', (None, None))

        if step_numeral == 1:
            """
            Basic Form: Application & File Uploader
            """
            return self.cleaned_data
        if step_numeral == 2:
            """
            Basic Form + Mapping Fields
            """
            return self.cleaned_data

        if step_numeral == 3:
            pass  # end-user is previewing


class ExportForm(forms.Form):
    corp_app = forms.ModelChoiceField(
                label=_('Corp Application'),
                queryset=CorpApp.objects.all())
    passcode = forms.CharField(label=_("Type Your Password"),
                               widget=forms.PasswordInput(render_value=False))

    def __init__(self, *args, **kwargs):
        self.user = kwargs.pop('user', '')
        super(ExportForm, self).__init__(*args, **kwargs)

    def clean_passcode(self):
        value = self.cleaned_data['passcode']

        if not self.user.check_password(value):
            raise forms.ValidationError(_("Invalid password."))
        return value



from django.db import models

from tendenci.core.robots.managers import RobotManager


STATUS_CHOICES = (('active','Active'),('inactive','Inactive'),)

class Robot(models.Model):
    name = models.CharField(max_length=150)
    url = models.URLField()
    version = models.CharField(max_length=50)
    status = models.BooleanField(default=True)
    status_detail = models.CharField(max_length=50, choices=STATUS_CHOICES,default='active')
    
    objects = RobotManager()
    
    def __unicode__(self):
        return self.name


from django.conf import settings

from tendenci.core.registry import site
from tendenci.core.registry.base import PeopleRegistry, lazy_reverse
from tendenci.apps.user_groups.models import Group


class GroupRegistry(PeopleRegistry):
    version = '1.0'
    author = 'Schipul - The Web Marketing Company'
    author_email = 'programmers@schipul.com'
    description = 'User Groups.'
    icon = '%simages/icons/groups-color-64x64.png' % settings.STATIC_URL
    
    event_logs = {
        'group':{
            'base':('160000','339999'),
            'add':('161000','339999'),
            'edit':('162000','339999'),
            'delete':('163000','339999'),
            'search':('164000','339999'),
            'view':('165000','339999'),
        },
        'groupmembership':{
            'base':('220000','00CCFF'),
            'add':('221000','00CCFF'),
            'edit':('222000','00CCFF'),
            'delete':('223000','00CCFF'),
            'search':('224000','00CCFF'),
            'view':('225000','00CCFF'),
        }
    }
    
    url = {
        'search': lazy_reverse('groups'),
        'add': lazy_reverse('group.add'),
    }

site.register(Group, GroupRegistry)



from tendenci.apps.boxes.models import Box
from tendenci.core.perms.forms import TendenciBaseForm
from django import forms
from tinymce.widgets import TinyMCE

class BoxForm(TendenciBaseForm):
    content = forms.CharField(required=False,
        widget=TinyMCE(attrs={'style':'width:100%'}, 
        mce_attrs={'storme_app_label':Box._meta.app_label, 
        'storme_model':Box._meta.module_name.lower()}))

    status_detail = forms.ChoiceField(
        choices=(('active','Active'),('inactive','Inactive'),))

    class Meta:
        model = Box
        fields = (
            'title',
            'content',
            'tags',
            'allow_anonymous_view',
            'user_perms',
            'member_perms',
            'group_perms',
            'status',
            'status_detail',
            )

    def __init__(self, *args, **kwargs): 
        super(BoxForm, self).__init__(*args, **kwargs)
        if self.instance.pk:
            self.fields['content'].widget.mce_attrs['app_instance_id'] = self.instance.pk
        else:
            self.fields['content'].widget.mce_attrs['app_instance_id'] = 0


from django.template import Library, Node, Variable
from tendenci.core.categories.models import Category

register = Library()

class GetCategoryForObjectNode(Node):
    def __init__(self, object, context):
        self.object = Variable(object)
        self.context = context
    def render(self, context):
        if not self.context: self.context = 'category'
        if not self.object: 
            context[self.context] = ''
            return ''  
        object = self.object.resolve(context)  
        category = Category.objects.get_for_object(object, 'category')
        if category: context[self.context] = category
        else: context[self.context] = ''
        return ''

@register.tag
def get_category_for_object(parser, token):
    """
        {% get_category_for_object object %}
    """
    bits  = token.split_contents()
    
    try: object = bits[1]
    except: object = None

    try: context = bits[3]
    except: context = None 
        
    return GetCategoryForObjectNode(object, context)

class GetSubCategoryForObjectNode(Node):
    def __init__(self, object, context):
        self.object = Variable(object)
        self.context = context
    def render(self, context):
        if not self.context: self.context = 'sub_category'
        if not self.object: 
            context[self.context] = ''
            return '' 
        object = self.object.resolve(context)  
        category = Category.objects.get_for_object(object, 'sub_category')
        if category: context[self.context] = category
        else: context[self.context] = ''
        return ''

@register.tag
def get_sub_category_for_object(parser, token):
    """
        {% get_sub_category_for_object object as sub_category %}
    """
    bits  = token.split_contents()
    
    try: object = bits[1]
    except: object = None
    
    try: context = bits[3]
    except: context = None 
    
    return GetSubCategoryForObjectNode(object, context)



from datetime import datetime
from dateutil.parser import parse as dt_parse

from django.contrib.auth.models import User

from celery.task import Task
from celery.registry import tasks

from tendenci.addons.locations.models import Location
from tendenci.addons.locations.importer.utils import parse_locs_from_csv


class ImportLocationsTask(Task):

    def run(self, user, file_path, fields, **kwargs):

        location_fields = [f.name for f in Location._meta.fields]

        #get parsed membership dicts
        imported = []
        locs, stats = parse_locs_from_csv(file_path, fields)

        for m in locs:
            if not m['skipped']:
                # Create Location object
                obj_dict = {}
                for key in m.keys():
                    if key in location_fields:
                        obj_dict[key] = m[key]

                # Add other fields
                obj_dict['location_name'] = m['locationname']
                obj_dict['creator'] = user
                obj_dict['creator_username'] = user.username
                obj_dict['owner'] = user
                obj_dict['owner_username'] = user.username

                # TODO: Set permissions properly
                obj_dict['allow_user_view'] = False
                obj_dict['allow_member_view'] = False
                obj_dict['allow_anonymous_edit'] = False
                obj_dict['allow_user_edit'] = False
                obj_dict['allow_member_edit'] = False

                new_location = Location.objects.create(**obj_dict)
 
                # append to imported list
                imported.append(new_location)

        return imported, stats




from django.core.management.base import BaseCommand, CommandError
from django.db.models.loading import get_model


class Command(BaseCommand):
    args = '<ics_pk>'
    help = "Runs an export task for the specified model."

    def handle(self, *args, **options):
        try:
            from tendenci.addons.events.ics.models import ICS
            from tendenci.addons.events.tasks import EventsICSTask
            
            if args:
                try:
                    ics = ICS.objects.get(pk=int(args[0]))
                except Export.DoesNotExist:
                    raise CommandError('ICS not specified')
                 
                self.stdout.write('Started compiling ics file...')
    
                
                result = EventsICSTask()
                response = result.run(ics=ics)
    
                ics.status = "completed"
                ics.result = response
                ics.save()
    
                self.stdout.write('Successfully completed ics file.')
            else:
                raise CommandError('ICS args not specified')
        except ImportError:
            print "Cannot precreate ics because events app is not installed."



from django.core.urlresolvers import reverse
from django.utils.functional import lazy
from django.contrib.admin import site as admin_site
from django.utils.translation import ugettext_lazy as _

from tendenci.core.registry.exceptions import *

lazy_reverse = lazy(reverse, str)


class FieldDict(dict):
    """
    Custom Field dict that will allow unicode
    representation based on a models verbose_name
    """
    def __unicode__(self):
        return self['verbose_name_plural'].lower()


class DeclarativeMetaclass(type):
    """
    Metaclass for app registry

    Loops through available fields and assigns them to
    a FieldDict to be used in templates or python code
    """
    def __new__(cls, name, bases, attrs):
        attrs['fields'] = FieldDict()
        allowed_fields = [
            '__doc__',
            'version',
            'author',
            'author_email',
            'description',
            'icon',
            'url',
            'packages',
            'event_logs'
        ]

        if 'app_registry' in attrs['__module__']:
            for field_name, item in attrs.items():
                if field_name not in ['fields', '__module__']:
                    if field_name not in allowed_fields:
                        exception = 'Registry field %s not allowed. '\
                                    'The following fields are allowed: %s'
                        raise FieldNotAllowed(_(exception) % (
                            field_name,
                            allowed_fields
                        ))
                    field = attrs.pop(field_name)
                    attrs['fields'][field_name] = field

        return super(DeclarativeMetaclass, cls).__new__(cls, name, bases, attrs)


class Registry(object):
    """
    Base registry class for core and plugin applications

    Example Registry:

    from tendenci.core.registry import site
    from tendenci.core.registry.base import CoreRegistry, lazy_reverse
    from tendenci.addons.models import ExmpleModel

    class ExampleRegistry(CoreRegistry):
        version = '1.0'
        author = 'Glen Zangirolami'
        author_email = 'glenbot@gmail.com'
        description = 'Create staff pages easily with photo, position, bio and more ..'

        url = {
            'add': lazy_reverse('page.add'),
            'search': lazy_reverse('page.search'),
        }

    site.register(ExampleModel, ExampleRegistry)
    """
    __metaclass__ = DeclarativeMetaclass

    def __init__(self, model):
        self.model = model
        self.has_admin = self._has_admin()

        # provide the model in case extra parameters are needed
        self.fields['model'] = self.model

        # transfer verbose names for easy access
        self.fields['verbose_name'] = unicode(self.model._meta.verbose_name)
        self.fields['verbose_name_plural'] = unicode(self.model._meta.verbose_name_plural)

        # default url patterns
        if 'url' not in self.fields:
            self.fields['url'] = self._get_url()

        # default icon path
        if 'icon' not in self.fields:
            self.fields['icon'] = self._get_icon()

        if 'packages' in self.fields:
            if not isinstance(self.fields['packages'], tuple):
                raise FieldError(_('Registry field packages must be of type tuple'))
            self._test_packages(self.fields['packages'])

    def _test_packages(self, packages):
        """
        Attempts to import the packages listed in the plugin registry
        Raises an error if the package cannot be imported
        """
        for package in packages:
            try:
                __import__(package)
            except:
                exception = 'Plugin dependency package `%(package)s` could not be imported. ' \
                            'Try `pip install %(package)s`?'
                raise PackageImportError(_(exception) % {
                    'package': package,
                })

    def _get_icon(self):
        """
        Autogenerate the location for the apps icon
        """
        # TODO: find a more global area to set this URL path
        default_plugin_url = 'plugin-media'
        return '/%s/%s/images/icon.png' % (
            default_plugin_url,
            unicode(self.model._meta.verbose_name_plural.lower())
        )

    def _get_url(self):
        """
        Autogenetate the URLs that will exist in the apps urls dict

        Example from template::

            {{ app.url.add }}
            {{ app.url.search }}
            {{ app.url.list }}

        Example in python::

            app['url']['add']
            app['url']['search']
            app['url']['list']
        """
        url = {
            'add': '',
            'search': '',
            'list': '',
        }
        admin_add = 'admin:%s_%s_add' % (
            self.model._meta.app_label,
            self.model._meta.module_name,
        )
        admin_index = 'admin:%s_%s_changelist' % (
            self.model._meta.app_label,
            self.model._meta.module_name,
        )
        if self.has_admin:
            url.update({
                'add': self._reverse(admin_add),
                'search': self._reverse(admin_index),
                'list': self._reverse(admin_index)
            })
        return url

    def _has_admin(self):
        """
        Tests for django admin site registration
        """
        for model in admin_site._registry.keys():
            if self.model is model:
                return True
        return False

    def _reverse(self, namespace):
        """
        Lazy url resolver
        """
        return lazy_reverse(namespace)


class CoreRegistry(Registry):
    """
    Registry for core applications
    """
    def __init__(self, model):
        super(CoreRegistry, self).__init__(model)

        # application type
        self.fields['app_type'] = 'core'


class AppRegistry(Registry):
    """
    Registry for addon applications
    """
    def __init__(self, model):
        super(AppRegistry, self).__init__(model)

        # application type
        self.fields['app_type'] = 'addon'


class PeopleRegistry(Registry):
    """
    Registry for people applications
    """
    def __init__(self, model):
        super(PeopleRegistry, self).__init__(model)

        # application type
        self.fields['app_type'] = 'people'


class LogRegistry(Registry):
    """
    Registry for event log associated applications that are not
    classified as core, plugin or people.
    """

    def __init__(self, model):
        super(LogRegistry, self).__init__(model)

        # application type
        self.fields['app_type'] = 'log'



from django.db import models
from django.contrib.auth.models import User
from django.utils.translation import ugettext_lazy as _
from django.utils.safestring import mark_safe
from django.contrib.contenttypes.models import ContentType

from tendenci.core.event_logs.models import EventLog
from tendenci.apps.entities.models import Entity
from tendenci.core.versions.models import Version


# Abstract base class for authority fields
class TendenciBaseModel(models.Model):
    # authority fields
    allow_anonymous_view = models.BooleanField(_("Public can view"), default=True)
    allow_user_view = models.BooleanField(_("Signed in user can view"))
    allow_member_view = models.BooleanField()
    allow_user_edit = models.BooleanField(_("Signed in user can change"))
    allow_member_edit = models.BooleanField()
    entity = models.ForeignKey(Entity, blank=True, null=True, default=None,
        on_delete=models.SET_NULL, related_name="%(app_label)s_%(class)s_entity")
    create_dt = models.DateTimeField(_("Created On"), auto_now_add=True)
    update_dt = models.DateTimeField(_("Last Updated"), auto_now=True)
    creator = models.ForeignKey(User, null=True, default=None, on_delete=models.SET_NULL,
        related_name="%(app_label)s_%(class)s_creator", editable=False)
    creator_username = models.CharField(max_length=50)
    owner = models.ForeignKey(User, null=True, default=None, on_delete=models.SET_NULL,
        related_name="%(app_label)s_%(class)s_owner")
    owner_username = models.CharField(max_length=50)
    status = models.BooleanField("Active", default=True)
    status_detail = models.CharField(max_length=50, default='active')

    @property
    def opt_app_label(self):
        return self._meta.app_label

    @property
    def opt_module_name(self):
        return self._meta.module_name

    def content_type(self):
        return ContentType.objects.get_for_model(self)

    def content_type_id(self):
        return ContentType.objects.get_for_model(self).pk

    @property
    def obj_perms(self):
        from tendenci.core.perms.fields import has_groups_perms
        t = '<span class="perm-%s">%s</span>'
 
        if self.allow_anonymous_view:
            value = t % ('public','Public')
        elif self.allow_user_view:
            value = t % ('users','Users')
        elif self.allow_member_view:
            value = t % ('members','Members')
        elif has_groups_perms(self):
            value = t % ('groups','Groups')
        else:
            value = t % ('private','Private')

        return mark_safe(value)

    @property
    def obj_status(obj):
        t = '<span class="status-%s">%s</span>'

        if obj.status:
            if obj.status_detail == 'paid - pending approval':
                value = t % ('pending', obj.status_detail.capitalize())
            else:
                value = t % (obj.status_detail, obj.status_detail.capitalize())
        else:
            value = t % ('inactive', 'Inactive')

        return mark_safe(value)

    class Meta:
        abstract = True

    def save(self, *args, **kwargs):
        if self.pk:
            log = kwargs.get('log', True)
            if log:
                application = self.__module__
                EventLog.objects.log(instance=self, application=application)

            # Save a version of this content.
            try:
                Version.objects.save_version(self.__class__.objects.get(pk=self.pk), self)
            except Exception as e:
                print "version error: ", e

        if "log" in kwargs:
            kwargs.pop('log')
        super(TendenciBaseModel, self).save(*args, **kwargs)

    def delete(self, *args, **kwargs):
        if self.pk:
            log = kwargs.get('log', True)
            if log:
                application = self.__module__
                EventLog.objects.log(instance=self, application=application)
        if "log" in kwargs:
            kwargs.pop('log')
        super(TendenciBaseModel, self).delete(*args, **kwargs)



import uuid
import re
from datetime import datetime, timedelta
from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.contrib.auth.models import User
from django.contrib.contenttypes import generic
from django.core.urlresolvers import reverse

from tagging.fields import TagField
from timezones.fields import TimeZoneField
from tinymce import models as tinymce_models
from tendenci.core.meta.models import Meta as MetaTags
from tendenci.core.base.fields import SlugField
from tendenci.core.perms.models import TendenciBaseModel 
from tendenci.core.perms.object_perms import ObjectPermission
from tendenci.core.categories.models import CategoryItem
from tendenci.apps.invoices.models import Invoice
from tendenci.core.site_settings.utils import get_setting

from tendenci.addons.directories.module_meta import DirectoryMeta
from tendenci.addons.directories.managers import DirectoryManager
from tendenci.addons.directories.choices import ADMIN_DURATION_CHOICES
from tendenci.libs.boto_s3.utils import set_s3_file_permission


def file_directory(instance, filename):
    filename = re.sub(r'[^a-zA-Z0-9._]+', '-', filename)
    return 'directories/%d/%s' % (instance.id, filename)

class Directory(TendenciBaseModel):
 
    guid = models.CharField(max_length=40)
    slug = SlugField(_('URL Path'), unique=True)
    timezone = TimeZoneField(_('Time Zone'))
    headline = models.CharField(max_length=200, blank=True)
    summary = models.TextField(blank=True)
    body = tinymce_models.HTMLField()
    source = models.CharField(max_length=300, blank=True)
    logo = models.FileField(max_length=260, upload_to=file_directory, 
                            help_text=_('Company logo. Only jpg, gif, or png images.'), 
                            blank=True)
    
    first_name = models.CharField(_('First Name'), max_length=100, blank=True)
    last_name = models.CharField(_('Last Name'), max_length=100, blank=True)
    address = models.CharField(_('Address'), max_length=100, blank=True)
    address2 = models.CharField(_('Address 2'), max_length=100, blank=True)
    city = models.CharField(_('City'), max_length=50, blank=True)
    state = models.CharField(_('State'), max_length=50, blank=True)
    zip_code = models.CharField(_('Zip Code'), max_length=50, blank=True)
    country = models.CharField(_('Country'), max_length=50, blank=True)
    phone = models.CharField(max_length=50, blank=True)
    phone2 = models.CharField(_('Phone 2'), max_length=50, blank=True)
    fax = models.CharField(_('Fax'), max_length=50, blank=True)
    email = models.CharField(_('Email'), max_length=120, blank=True)
    email2 = models.CharField(_('Email 2'), max_length=120, blank=True)
    website = models.CharField(max_length=300, blank=True)
    
    renewal_notice_sent = models.BooleanField(default=False)
    list_type = models.CharField(_('List Type'), max_length=50, blank=True)
    requested_duration = models.IntegerField(_('Requested Duration'), default=0)
    pricing = models.ForeignKey('DirectoryPricing', null=True)
    activation_dt = models.DateTimeField(_('Activation Date/Time'), null=True, blank=True)
    expiration_dt = models.DateTimeField(_('Expiration Date/Time'), null=True, blank=True)
    invoice = models.ForeignKey(Invoice, blank=True, null=True) 
    payment_method = models.CharField(_('Payment Method'), max_length=50, blank=True)

    syndicate = models.BooleanField(_('Include in RSS feed'), default=True)
    design_notes = models.TextField(_('Design Notes'), blank=True)
    admin_notes = models.TextField(_('Admin Notes'), blank=True)
    tags = TagField(blank=True)
   
    # for podcast feeds
    enclosure_url = models.CharField(_('Enclosure URL'), max_length=500, blank=True)
    enclosure_type = models.CharField(_('Enclosure Type'), max_length=120, blank=True)
    enclosure_length = models.IntegerField(_('Enclosure Length'), default=0)
    
    # html-meta tags
    meta = models.OneToOneField(MetaTags, null=True)

    categories = generic.GenericRelation(CategoryItem,
                                          object_id_field="object_id",
                                          content_type_field="content_type")
    perms = generic.GenericRelation(ObjectPermission,
                                          object_id_field="object_id",
                                          content_type_field="content_type")

    objects = DirectoryManager()

    class Meta:
        permissions = (("view_directory","Can view directory"),)
        verbose_name = "Directory"
        verbose_name_plural = "Directories"

    def get_meta(self, name):
        """
        This method is standard across all models that are
        related to the Meta model.  Used to generate dynamic
        methods coupled to this instance.
        """    
        return DirectoryMeta().get_meta(self, name)
    
    @models.permalink
    def get_absolute_url(self):
        return ("directory", [self.slug])

    @models.permalink
    def get_renew_url(self):
        return ("directory.renew", [self.id])

    def __unicode__(self):
        return self.headline

    def save(self, *args, **kwargs):
        if not self.id:
            self.guid = str(uuid.uuid1())

        super(Directory, self).save(*args, **kwargs)
        if self.logo:
            if self.is_public():
                set_s3_file_permission(self.logo.name, public=True)
            else:
                set_s3_file_permission(self.logo.name, public=False)

    def is_public(self):
        return all([self.allow_anonymous_view,
                self.status,
                self.status_detail in ['active']])

    def get_logo_url(self):
        if not self.logo:
            return ''

        if self.is_public():
            return self.logo.url

        return reverse('directory.logo', args=[self.id])

    # Called by payments_pop_by_invoice_user in Payment model.
    def get_payment_description(self, inv):
        """
        The description will be sent to payment gateway and displayed on invoice.
        If not supplied, the default description will be generated.
        """
        return 'Tendenci Invoice %d for Directory: %s (%d).' % (
            inv.id,
            self.headline,
            inv.object_id,
        )

    def make_acct_entries(self, user, inv, amount, **kwargs):
        """
        Make the accounting entries for the directory sale
        """
        from tendenci.apps.accountings.models import Acct, AcctEntry, AcctTran
        from tendenci.apps.accountings.utils import make_acct_entries_initial, make_acct_entries_closing

        ae = AcctEntry.objects.create_acct_entry(user, 'invoice', inv.id)
        if not inv.is_tendered:
            make_acct_entries_initial(user, ae, amount)
        else:
            # payment has now been received
            make_acct_entries_closing(user, ae, amount)

            # #CREDIT directory SALES
            acct_number = self.get_acct_number()
            acct = Acct.objects.get(account_number=acct_number)
            AcctTran.objects.create_acct_tran(user, ae, acct, amount*(-1))

    def get_acct_number(self, discount=False):
        if discount:
            return 464400
        else:
            return 404400

    def auto_update_paid_object(self, request, payment):
        """
        Update the object after online payment is received.
        """
        if not request.user.profile.is_superuser:
            self.status_detail = 'paid - pending approval'
            self.save()

    def age(self):
        return datetime.now() - self.create_dt

    @property
    def category_set(self):
        items = {}
        for cat in self.categories.select_related('category__name', 'parent__name'):
            if cat.category:
                items["category"] = cat.category
            elif cat.parent:
                items["sub_category"] = cat.parent
        return items
    
    def renew_window(self):
        days = get_setting('module', 'directories', 'renewaldays')
        days = int(days)
        if datetime.now() + timedelta(days) > self.expiration_dt:
            return True
        else:
            return False

class DirectoryPricing(models.Model):
    guid = models.CharField(max_length=40)
    duration = models.IntegerField(blank=True, choices=ADMIN_DURATION_CHOICES)
    regular_price =models.DecimalField(max_digits=15, decimal_places=2, blank=True, default=0)
    premium_price = models.DecimalField(max_digits=15, decimal_places=2, blank=True, default=0)
    regular_price_member = models.DecimalField(max_digits=15, decimal_places=2, blank=True, default=0)
    premium_price_member = models.DecimalField(max_digits=15, decimal_places=2, blank=True, default=0)
    show_member_pricing = models.BooleanField()
    create_dt = models.DateTimeField(auto_now_add=True)
    update_dt = models.DateTimeField(auto_now=True)
    creator = models.ForeignKey(User, related_name="directory_pricing_creator",  null=True, on_delete=models.SET_NULL)
    creator_username = models.CharField(max_length=50, null=True)
    owner = models.ForeignKey(User, related_name="directory_pricing_owner", null=True, on_delete=models.SET_NULL)
    owner_username = models.CharField(max_length=50, null=True)
    status = models.BooleanField(default=True)

    class Meta:
        permissions = (("view_directorypricing", "Can view directory pricing"),)
    
    def __unicode__(self):
        return "Directory Pricing %s" % (self.id)

    def save(self, user=None, *args, **kwargs):
        if not self.id:
            self.guid = str(uuid.uuid1())
            if user and user.id:
                self.creator=user
                self.creator_username=user.username
        if user and user.id:
            self.owner=user
            self.owner_username=user.username
        if not self.regular_price: self.regular_price = 0
        if not self.premium_price: self.premium_price = 0
            
        super(DirectoryPricing, self).save(*args, **kwargs)




from datetime import datetime
from os.path import join
from uuid import uuid4
from decimal import Decimal

from django import forms
from django.core.files.storage import FileSystemStorage
from django.utils.importlib import import_module
from django.utils.translation import ugettext_lazy as _
from django.template.defaultfilters import slugify
from django.utils.safestring import mark_safe
from django.core.files.storage import default_storage

from tendenci.core.site_settings.utils import get_setting
from tendenci.core.payments.models import PaymentMethod
from tinymce.widgets import TinyMCE
from tendenci.core.perms.forms import TendenciBaseForm
from captcha.fields import CaptchaField
from tendenci.apps.user_groups.models import Group
from tendenci.core.base.utils import get_template_list
from tendenci.core.base.fields import EmailVerificationField

from tendenci.addons.recurring_payments.fields import BillingCycleField
from tendenci.addons.recurring_payments.widgets import BillingCycleWidget, BillingDateSelectWidget
from tendenci.apps.forms_builder.forms.models import FormEntry, FieldEntry, Field, Form, Pricing
from tendenci.apps.forms_builder.forms.settings import FIELD_MAX_LENGTH, UPLOAD_ROOT

template_choices = [('default.html','Default')]
template_choices += get_template_list()

#fs = FileSystemStorage(location=UPLOAD_ROOT)

class FormForForm(forms.ModelForm):

    class Meta:
        model = FormEntry
        exclude = ("form", "entry_time", "entry_path", "payment_method", "pricing", "creator")
    
    def __init__(self, form, user, *args, **kwargs):
        """
        Dynamically add each of the form fields for the given form model 
        instance and its related field model instances.
        """
        self.user = user
        self.form = form
        self.form_fields = form.fields.visible().order_by('position')
        super(FormForForm, self).__init__(*args, **kwargs)

        for field in self.form_fields:
            field_key = "field_%s" % field.id
            if "/" in field.field_type:
                field_class, field_widget = field.field_type.split("/")
            else:
                field_class, field_widget = field.field_type, None

            if field.field_type == 'EmailVerificationField':
                field_class = EmailVerificationField
            else:
                field_class = getattr(forms, field_class)
            field_args = {"label": mark_safe(field.label), "required": field.required}
            arg_names = field_class.__init__.im_func.func_code.co_varnames
            if "max_length" in arg_names:
                field_args["max_length"] = FIELD_MAX_LENGTH
            if "choices" in arg_names:
                choices = field.choices.split(",")
                field_args["choices"] = zip(choices, choices)
            if "initial" in arg_names:
                default = field.default.lower()
                if field_class == "BooleanField":
                    if default == "checked" or default == "true" or \
                        default == "on" or default == "1":
                            default = True
                    else:
                        default = False
                field_args["initial"] = field.default

            if field_widget is not None:
                module, widget = field_widget.rsplit(".", 1)
                field_args["widget"] = getattr(import_module(module), widget)
            
            self.fields[field_key] = field_class(**field_args)

            self.fields[field_key].widget.attrs['title'] = field.label

        # include pricing options if any
        if (self.form.custom_payment or self.form.recurring_payment) and self.form.pricing_set.all():

            currency_symbol = get_setting('site', 'global', 'currencysymbol')

            pricing_options = []
            for pricing in self.form.pricing_set.all():

                if pricing.price == None:
                    pricing_options.append(
                        (pricing.pk, mark_safe(
                            '<input type="text" class="custom-price" name="custom_price_%s" value="%s"/> %s' % 
                            (pricing.pk, self.data.get('custom_price_%s' %pricing.pk, unicode()), pricing.label)))
                    )
                else:
                    if self.form.recurring_payment:
                        pricing_options.append(
                            (pricing.pk, '%s%s per %s %s - %s' % (currency_symbol, pricing.price,
                                                                  pricing.billing_frequency, pricing.billing_period,
                                                                  pricing.label))
                        )
                    else:
                        pricing_options.append(
                            (pricing.pk, '%s%s %s' % (currency_symbol, pricing.price, pricing.label))
                        )

            self.fields['pricing_option'] = forms.ChoiceField(
                label=_('Pricing'),
                choices = pricing_options,
                widget=forms.RadioSelect
            )

            self.fields['payment_option'] = forms.ModelChoiceField(
                    label=_('Payment Method'),
                    empty_label=None,
                    queryset=self.form.payment_methods.all(),
                    widget=forms.RadioSelect,
                    initial=1,
                )
        
        if not self.user.is_authenticated() and get_setting('site', 'global', 'captcha'): # add captcha if not logged in
            self.fields['captcha'] = CaptchaField(label=_('Type the code below'))


    def clean_pricing_option(self):
        pricing_pk = int(self.cleaned_data['pricing_option'])
        pricing_option = self.form.pricing_set.get(pk=pricing_pk)
        custom_price = self.data.get('custom_price_%s' % pricing_pk)

        # if not price set
        if not pricing_option.price:
            # then price is custom

            if not custom_price:  # custom price has a value
                raise forms.ValidationError("Please set your price.")

            try:  # custom price is valid amount
                custom_price = Decimal(custom_price)
            except ValueError:
                raise forms.ValidationError("Price must be a valid amount")
        
        self.cleaned_data['custom_price'] = custom_price
        return pricing_option

    def save(self, **kwargs):
        """
        Create a FormEntry instance and related FieldEntry instances for each 
        form field.
        """
        entry = super(FormForForm, self).save(commit=False)
        entry.form = self.form
        entry.entry_time = datetime.now()
        entry.save()
        for field in self.form_fields:
            field_key = "field_%s" % field.id
            value = self.cleaned_data[field_key]
            if value and self.fields[field_key].widget.needs_multipart_form:
                value = default_storage.save(join("forms", str(uuid4()), value.name), value)
            # if the value is a list convert is to a comma delimited string
            if isinstance(value,list):
                value = ','.join(value)
            if not value: value=''
            field_entry = FieldEntry(field_id = field.id, entry=entry, value = value)
            if self.user.is_authenticated():
                field_entry.save(user = self.user)
            else:
                field_entry.save()

        # save selected pricing and payment method if any
        if (self.form.custom_payment or self.form.recurring_payment) and self.form.pricing_set.all():
            entry.payment_method = self.cleaned_data['payment_option']
            entry.pricing = self.cleaned_data['pricing_option']
            entry.save()
            
        return entry
        
    def email_to(self):
        """
        Return the value entered for the first field of type EmailField.
        """
        for field in self.form_fields:
            field_class = field.field_type.split("/")[0]
            if field_class == "EmailField":
                return self.cleaned_data["field_%s" % field.id]
        return None
        
        
class FormAdminForm(TendenciBaseForm):
    status_detail = forms.ChoiceField(
        choices=(('draft','Draft'),('published','Published'),))

    intro = forms.CharField(required=False,
        widget=TinyMCE(attrs={'style':'width:100%'}, 
        mce_attrs={'storme_app_label':Form._meta.app_label, 
        'storme_model':Form._meta.module_name.lower()}))

    response = forms.CharField(required=False, label='Confirmation Text', 
        widget=TinyMCE(attrs={'style':'width:100%'}, 
        mce_attrs={'storme_app_label':Form._meta.app_label, 
        'storme_model':Form._meta.module_name.lower()}))

    template = forms.ChoiceField(choices=template_choices)

    class Meta:
        model = Form
        fields = ('title',
                  'slug',
                  'intro',
                  'response',
                  'template',
                  'send_email', # removed per ed's request, added back per Aaron's request 2011-10-14
                  'email_text',
                  'subject_template',
                  'completion_url',
                  'email_from',
                  'email_copies',
                  'user_perms',
                  'member_perms',
                  'group_perms',
                  'allow_anonymous_view',
                  'status',
                  'status_detail',
                  'custom_payment',
                  'recurring_payment',
                  'payment_methods',
                 )

    def __init__(self, *args, **kwargs): 
        super(FormAdminForm, self).__init__(*args, **kwargs)
        if self.instance.pk:
            self.fields['intro'].widget.mce_attrs['app_instance_id'] = self.instance.pk
            self.fields['response'].widget.mce_attrs['app_instance_id'] = self.instance.pk
        else:
            self.fields['intro'].widget.mce_attrs['app_instance_id'] = 0
            self.fields['response'].widget.mce_attrs['app_instance_id'] = 0


    def clean_slug(self):
        slug = slugify(self.cleaned_data['slug'])
        i = 0
        while True:
            if i > 0:
                if i > 1:
                    slug = slug.rsplit("-", 1)[0]
                slug = "%s-%s" % (slug, i)
            match = Form.objects.filter(slug=slug)
            if self.instance:
                match = match.exclude(pk=self.instance.pk)
            if not match:
                break
            i += 1
        return slug
        
class FormForm(TendenciBaseForm):
    # from django.utils.safestring import mark_safe

    status_detail = forms.ChoiceField(
        choices=(('draft','Draft'),('published','Published'),))
    custom_payment = forms.BooleanField(label=_('Take Payment'), required=False)

    # payment_method_choices = list(PaymentMethod.objects.values_list('id','human_name'))
    # payment_method_choices.append((0, mark_safe('<input type="text" name="custom-payment-method" />')))
    # payment_methods = forms.MultipleChoiceField(
    #     choices=payment_method_choices,
    #     widget=forms.CheckboxSelectMultiple(),
    #     required=False
    # )

    payment_methods = forms.ModelMultipleChoiceField(
            queryset=PaymentMethod.objects.all(),
            widget=forms.CheckboxSelectMultiple,
            required=False,
            initial=[1,2,3]
        )
    
    class Meta:
        model = Form
        fields = ('title',
                  'slug',
                  'intro',
                  'response',
                  'send_email', # removed per ed's request, added back per Aaron's request 2011-10-14
                  'email_text',
                  'completion_url',
                  'subject_template',
                  'email_from',
                  'email_copies',
                  'custom_payment',
                  'recurring_payment',
                  'payment_methods',
                  'user_perms',
                  'member_perms',
                  'group_perms',
                  'allow_anonymous_view',
                  'status',
                  'status_detail',
                 )
        fieldsets = [('Form Information', {
                        'fields': [ 'title',
                                    'slug',
                                    'intro',
                                    'response',
                                    'completion_url',
                                    'subject_template',
                                    'email_from',
                                    'email_copies',
                                    'send_email',
                                    'email_text',
                                    ],
                        'legend': ''
                        }),
                    ('Permissions', {
                        'fields': [ 'allow_anonymous_view',
                                    'user_perms',
                                    'member_perms',
                                    'group_perms',
                                    ],
                        'classes': ['permissions'],
                        }),
                    ('Administrator Only', {
                        'fields': ['status',
                                    'status_detail'], 
                        'classes': ['admin-only'],
                    }),
                    ('Payments', {
                        'fields':['custom_payment','recurring_payment','payment_methods'],
                        'legend':''
                    }),]
                
    def __init__(self, *args, **kwargs):
        super(FormForm, self).__init__(*args, **kwargs)

        if not self.user.profile.is_superuser:
            if 'status' in self.fields:
                self.fields.pop('status')
            if 'status_detail' in self.fields:
                self.fields.pop('status_detail')

    def clean_slug(self):
        slug = slugify(self.cleaned_data['slug'])
        i = 0
        while True:
            if i > 0:
                if i > 1:
                    slug = slug.rsplit("-", 1)[0]
                slug = "%s-%s" % (slug, i)
            match = Form.objects.filter(slug=slug)
            if self.instance:
                match = match.exclude(pk=self.instance.pk)
            if not match:
                break
            i += 1
        return slug

class FormForField(forms.ModelForm):
    class Meta:
        model = Field
        exclude = ["position"]
    
    def clean_function_params(self):
        function_params = self.cleaned_data['function_params']
        clean_params = ''
        for val in function_params.split(','):
            clean_params = val.strip() + ',' + clean_params
        return clean_params[0:len(clean_params)-1]
        
    def clean(self):
        cleaned_data = self.cleaned_data
        field_function = cleaned_data.get("field_function")
        function_params = cleaned_data.get("function_params")
        field_type = cleaned_data.get("field_type")
        required = cleaned_data.get("required")
        
        if field_function == "GroupSubscription":
            if field_type != "BooleanField":
                raise forms.ValidationError("This field's function requires Checkbox as a field type")
            if not function_params:
                raise forms.ValidationError("This field's function requires at least 1 group specified.")
            else:
                for val in function_params.split(','):
                    try:
                        Group.objects.get(name=val)
                    except Group.DoesNotExist:
                        raise forms.ValidationError("The group \"%s\" does not exist" % (val))
                    
        if field_function != None and field_function.startswith("Email"):
            if field_type != "CharField":
                raise forms.ValidationError("This field's function requires Text as a field type")
        
        #unrequire the display only fields
        if field_type == "CharField/tendenci.apps.forms_builder.forms.widgets.Description":
            cleaned_data['required'] = False
        elif field_type == "CharField/tendenci.apps.forms_builder.forms.widgets.Header":
            cleaned_data['required'] = False
            
        return cleaned_data

        
class PricingForm(forms.ModelForm):
    billing_dt_select = BillingCycleField(label='When to bill',
                                          widget=BillingDateSelectWidget,
                                          help_text='It is used to determine the payment due date for each billing cycle')
    billing_cycle = BillingCycleField(label='How often to bill',
                                      widget=BillingCycleWidget)

    class Meta:
        model = Pricing
        fields = ('label',
                  'price',
                  'taxable',
                  'tax_rate',
                  'billing_cycle',
                  'billing_dt_select',
                  'has_trial_period',
                  'trial_period_days',
                 )
        fieldsets = [('Form Information', {
                        'fields': [ 'label',
                                    'price',
                                    'taxable',
                                    'tax_rate',
                                    'billing_cycle',
                                    'billing_dt_select',
                                    ],
                        'legend': ''
                        }),
                    ('Trial Period', {
                        'fields': [ 'has_trial_period',
                                    'trial_period_days',],
                        'legend': '',
                        }),]

    def __init__(self, *args, **kwargs):
        super(PricingForm, self).__init__(*args, **kwargs)
        # Setup initial values for billing_cycle and billing_dt_select
        # in order to have empty values for extra forms.
        if self.instance.pk:
            self.fields['billing_dt_select'].initial = [self.instance.num_days, 
                                                        self.instance.due_sore]
            self.fields['billing_cycle'].initial = [self.instance.billing_frequency, 
                                                    self.instance.billing_period]
        else:
            self.fields['billing_dt_select'].initial = [0, u'start']
            self.fields['billing_cycle'].initial = [1, u'month']
        
        # Add class for recurring payment fields
        self.fields['taxable'].widget.attrs['class'] = 'recurring-payment'
        self.fields['tax_rate'].widget.attrs['class'] = 'recurring-payment'
        self.fields['billing_cycle'].widget.attrs['class'] = 'recurring-payment'
        self.fields['billing_dt_select'].widget.attrs['class'] = 'recurring-payment'
        self.fields['has_trial_period'].widget.attrs['class'] = 'recurring-payment'
        self.fields['trial_period_days'].widget.attrs['class'] = 'recurring-payment'
    
    def save(self, **kwargs):
        pricing = super(PricingForm, self).save(**kwargs)
        if self.cleaned_data.get('billing_dt_select'):
            dt_select = self.cleaned_data.get('billing_dt_select').split(',')
            pricing.num_days = dt_select[0]
            pricing.due_sore = dt_select[1]
        if self.cleaned_data.get('billing_cycle'):
            cycle = self.cleaned_data.get('billing_cycle').split(',')
            pricing.billing_frequency = cycle[0]
            pricing.billing_period = cycle[1]
        #pricing.save()    
        return pricing
        
class BillingForm(forms.Form):
    first_name = forms.CharField(required=False)
    last_name = forms.CharField(required=False)
    company = forms.CharField(required=False)
    address = forms.CharField(required=False)
    city = forms.CharField(required=False)
    state = forms.CharField(required=False)
    zip_code = forms.CharField(required=False)
    country = forms.CharField(required=False)
    phone = forms.CharField(required=False)
    email = forms.CharField(required=False)



from django.core.management.base import BaseCommand

class Command(BaseCommand):
    """
    Populate invoices with creator IDs from registrations.
    """
    def handle(self, *args, **options):
        from tendenci.addons.events.models import Registration
        from tendenci.apps.invoices.models import Invoice

        registrations = Registration.objects.filter(creator__isnull=False)
        
        for registration in registrations:
            related_invoice = Invoice.objects.filter(id=registration.invoice_id)
            for invoice in related_invoice:
                invoice.creator_id = registration.creator_id
                invoice.save()
                print 'Invoice %s updated for Registraion %s' % (invoice.id, registration.id)


# settings - directoriespaymenttypes, directoriesrequirespayment
from datetime import datetime
from cStringIO import StringIO
from PIL import Image
from django.contrib.contenttypes.models import ContentType
from tendenci.addons.directories.models import DirectoryPricing
from tendenci.apps.invoices.models import Invoice
from tendenci.core.payments.models import Payment
from tendenci.core.site_settings.utils import get_setting
from tendenci.libs.storage import get_default_storage


def resize_s3_image(image_path, width=200, height=200):
    """
    Resize an image on s3. 
    The image_path is the relative path to the media storage.
    """
    storage = get_default_storage()
    f = storage.open(image_path)
    content = f.read()
    f.close()
    img = Image.open(StringIO(content))
    img.thumbnail((width,height),Image.ANTIALIAS)
    f = storage.open(image_path, 'w')
    img.save(f)
    f.close()

def get_duration_choices(user):
    currency_symbol = get_setting('site', 'global', 'currencysymbol')
    pricings = DirectoryPricing.objects.filter(status=True).order_by('duration')
    choices = []
    for pricing in pricings:
        if pricing.duration == 0:
            if user.profile.is_superuser:
                choice = (pricing.pk, 'Unlimited')
            else:
                continue
        else:
            if pricing.show_member_pricing and user.profile.is_member:
                prices = "%s%s(R)/%s(P)" % (currency_symbol,pricing.regular_price_member, 
                                            pricing.premium_price_member)
            else:
                prices = "%s%s(R)/%s(P)" % (currency_symbol, pricing.regular_price, 
                                    pricing.premium_price)
            choice = (pricing.pk, '%d days for %s' % (pricing.duration, prices))
        choices.append(choice)
        
    return choices

def get_payment_method_choices(user):
    if user.profile.is_superuser:
        return (('paid - check', 'User paid by check'),
                ('paid - cc', 'User paid by credit card'),
                ('Credit Card', 'Make online payment NOW'),)
    else:
        directory_payment_types = get_setting('module', 'directories', 'directoriespaymenttypes')
        if directory_payment_types:
            directory_payment_types_list = directory_payment_types.split(',')
            directory_payment_types_list = [item.strip() for item in directory_payment_types_list]
            
            return [(item, item) for item in directory_payment_types_list]
        else:
            return ()
  
def directory_set_inv_payment(user, directory, pricing): 
    if get_setting('module', 'directories', 'directoriesrequirespayment'):
        if not directory.invoice:
            inv = Invoice()
            inv.object_type = ContentType.objects.get(app_label=directory._meta.app_label, 
                                              model=directory._meta.module_name)
            inv.object_id = directory.id
            profile = user.get_profile()
            inv.title = "Directory Add Invoice"
            inv.bill_to = '%s %s' % (user.first_name, user.last_name)
            inv.bill_to_first_name = user.first_name
            inv.bill_to_last_name = user.last_name
            inv.bill_to_company = profile.company
            inv.bill_to_address = profile.address
            inv.bill_to_city = profile.city
            inv.bill_to_state = profile.state
            inv.bill_to_zip_code = profile.zipcode
            inv.bill_to_country = profile.country
            inv.bill_to_phone = profile.phone
            inv.bill_to_fax = profile.fax
            inv.bill_to_email = user.email
            inv.ship_to = inv.bill_to
            inv.ship_to_first_name = user.first_name
            inv.ship_to_last_name = user.last_name
            inv.ship_to_company = profile.company
            inv.ship_to_address = profile.address
            inv.ship_to_city = profile.city
            inv.ship_to_state = profile.state
            inv.ship_to_zip_code = profile.zipcode
            inv.ship_to_country = profile.country
            inv.ship_to_phone = profile.phone
            inv.ship_to_fax = profile.fax
            inv.ship_to_email = user.email
            inv.terms = "Due on Receipt"
            inv.due_date = datetime.now()
            inv.ship_date = datetime.now()
            inv.message = 'Thank You.'
            inv.status = True
            
            inv.total = get_directory_price(user, directory, pricing)
            inv.subtotal = inv.total
            inv.balance = inv.total
            inv.estimate = 1
            inv.status_detail = 'estimate'
            inv.save(user)
            
            # tender the invoice
            inv.tender(user)
            
            # update job
            directory.invoice = inv
            directory.save()
            
            if user.profile.is_superuser:
                if directory.payment_method in ['paid - cc', 'paid - check', 'paid - wire transfer']:
                    boo_inv = inv.tender(user) 
                    
                    # payment
                    payment = Payment()
                    boo = payment.payments_pop_by_invoice_user(user, inv, inv.guid)
                    payment.mark_as_paid()
                    payment.method = directory.payment_method
                    payment.save(user)
                    
                    # this will make accounting entry
                    inv.make_payment(user, payment.amount)
                    
            
def get_directory_price(user, directory, pricing):
    directory_price = 0
    if user.profile.is_member:
        if directory.list_type == 'regular':
            return  pricing.regular_price_member
        else:
            return pricing.premium_price_member
    else:
        if directory.list_type == 'regular':
            return  pricing.regular_price
        else:
            return pricing.premium_price
                



from django.db import models
from django.contrib.auth.models import User
from tendenci.apps.accountings.managers import AcctEntryManager, AcctTranManager

class Acct(models.Model):
    account_number = models.IntegerField(unique=True)
    description = models.TextField()
    type = models.CharField(max_length=5)

class AcctEntry(models.Model):
    source = models.TextField()
    object_id = models.IntegerField()
    entry_dt = models.DateTimeField(auto_now_add=True)
    exported = models.BooleanField(default=0)
    create_dt = models.DateTimeField(auto_now_add=True)
    creator = models.ForeignKey(User, related_name="accentry_creator",  null=True, on_delete=models.SET_NULL)
    creator_username = models.CharField(max_length=50, default='')
    owner = models.ForeignKey(User, related_name="accentry_owner", null=True, on_delete=models.SET_NULL)
    owner_username = models.CharField(max_length=50, default='')
    status = models.BooleanField(default=True)
    
    objects = AcctEntryManager()
    
    
class AcctTran(models.Model):
    acct_entry = models.ForeignKey(AcctEntry, related_name="accttran_acctentry")
    #account_number = models.IntegerField()
    account = models.ForeignKey(Acct, related_name="accttran_acct",  null=True)
    amount = models.DecimalField(max_digits=10, decimal_places=2, blank=True)
    cleared = models.BooleanField(default=False)
    create_dt = models.DateTimeField(auto_now_add=True)
    creator = models.ForeignKey(User, related_name="accttran_creator",  null=True, on_delete=models.SET_NULL)
    owner = models.ForeignKey(User, related_name="accttran_owner", null=True, on_delete=models.SET_NULL)
    status = models.BooleanField(default=True)
    
    objects = AcctTranManager()
    
    @property
    def debit(self):
        if self.amount > 0:
            return self.amount
        return 0
    
    @property
    def credit(self):
        if self.amount < 0:
            return abs(self.amount)
        return 0
    

    





from django.template import Library, TemplateSyntaxError, Variable

from tendenci.addons.recurring_payments.models import RecurringPayment

register = Library()


@register.inclusion_tag("recurring_payments/nav.html", takes_context=True)
def rp_nav(context, user, rp=None):
    context.update({
        "nav_object": rp,
        "user": user
    })
    return context


from tendenci.core.perms.managers import TendenciBaseManager

class PhotoSetManager(TendenciBaseManager):
    """
    Model Manager
    """
    pass

class PhotoManager(TendenciBaseManager):
    """
    Model Manager
    """
    pass
    



from django.core.cache import cache
from django.conf import settings
import pickle

REGISTRY_PRE_KEY = "registry"

def delete_reg_apps_cache():
    """Delete cache for all apps
    """
    keys = [settings.CACHE_PRE_KEY, REGISTRY_PRE_KEY, 'reg_apps']
    key = '.'.join(keys)
    cache.delete(key)

def cache_reg_apps(apps):
    """Caches the list of registered apps
    """
    keys = [settings.CACHE_PRE_KEY, REGISTRY_PRE_KEY, 'reg_apps']
    key = '.'.join(keys)
    
    value = apps.all_apps
    
    cache.set(key, value)

def get_reg_apps():
    """Gets all the registered apps from the cache
    """
    keys = [settings.CACHE_PRE_KEY, REGISTRY_PRE_KEY, 'reg_apps']
    key = '.'.join(keys)
    
    all_apps = cache.get(key)
    
    return all_apps



from tendenci.core.perms.managers import TendenciBaseManager
from tendenci.core.categories.models import Category


class DirectoryManager(TendenciBaseManager):
    """
    Model Manager
    """
    def get_categories(self, category=None):
        return Category.objects.get_for_model(self.model, category)


from django.conf.urls.defaults import patterns, url
from django.views.generic.simple import direct_to_template
from django.contrib.auth import views as auth_views

from tendenci.apps.registration.views import activate
from tendenci.apps.profiles.views import password_change, password_change_done
from tendenci.apps.accounts.forms import RegistrationCustomForm, PasswordResetForm, SetPasswordCustomForm
from tendenci.apps.accounts.views import register

urlpatterns = patterns('tendenci.apps',
                       # Activation keys get matched by \w+ instead of the more specific
                       # [a-fA-F0-9]{40} because a bad activation key should still get to the view;
                       # that way it can return a sensible "invalid key" message instead of a
                       # confusing 404.
                       url(r'^activate/(?P<activation_key>\w+)/$',
                           activate,
                           {'template_name': 'accounts/activate.html'},
                           name='registration_activate'),
                       url(r'^login/$',
                           'accounts.views.login',
                           {'template_name': 'accounts/login.html'},
                           name='auth_login'),
                       url(r'^logout/$',
                           auth_views.logout,
                           {'template_name': 'accounts/logout.html'},
                           name='auth_logout'),
                       url(r'^password/change/(?P<id>\d+)/$',
                           password_change,
                           name='auth_password_change'),
                       url(r'^password/change/done/(?P<id>\d+)/$',
                           password_change_done,
                           name='auth_password_change_done'),
                       url(r'^password/reset/$',
                           'accounts.views.password_reset',
                           name='auth_password_reset'),
                       url(r'^password/reset/confirm/(?P<uidb36>[0-9A-Za-z]+)-(?P<token>.+)/$',
                           auth_views.password_reset_confirm, {'set_password_form':SetPasswordCustomForm},
                           name='auth_password_reset_confirm'),
                       url(r'^password/reset/complete/$',
                           auth_views.password_reset_complete,
                           name='auth_password_reset_complete'),
                       url(r'^password/reset/done/$',
                           auth_views.password_reset_done,
                           name='auth_password_reset_done'),
                       url(r'^register/$',
                           register, {'form_class' : RegistrationCustomForm, 'template_name': 'accounts/registration_form.html'},
                           name='registration_register'),
                       url(r'^register/event/(?P<event_id>\d+)/$', register, {'form_class' : RegistrationCustomForm, 'template_name': 'accounts/registration_form.html'}, name='registration_event_register'),
                       url(r'^register/complete/$',
                           direct_to_template,
                           {'template': 'accounts/registration_complete.html'},
                           name='registration_complete'),
                       )



from tendenci.core.registry import site
from tendenci.core.registry.base import LogRegistry, lazy_reverse
from tendenci.core.payments.models import Payment


class PaymentRegistry(LogRegistry):
    # payments - PINK ORANGE base - complement is ????
    event_logs = {
        'payments':{
            'base':('280000', 'FF6666'), # base
            'add':('281000', 'FF6666'), # add
            'edit':('282000', 'FF6666'), # edit
            'delete':('283000', 'FF6666'), # delete
            'search':('284000', 'FF6666'), # search 
            'view':('285000', 'FF6666'), # view 
            'export':('286000', 'FF6666'), # export 
            'edit_credit_card_approved':('282101', 'FF6666'), # Edit - Credit card approved 
            'edit_credit_card_declined':('282102', 'FF6666'), # Edit - Credit card declined 
        },
    }

site.register(Payment, PaymentRegistry)



from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.contrib.contenttypes import generic

from tendenci.core.perms.object_perms import ObjectPermission
from tendenci.core.perms.models import TendenciBaseModel
from tendenci.apps.pages.models import Page
from tendenci.apps.navs.managers import NavManager

class Nav(TendenciBaseModel):
    class Meta:
        permissions = (("view_nav","Can view nav"),)
    
    title = models.CharField(max_length=100)
    description = models.TextField(blank=True, null=True)
    megamenu = models.BooleanField(default=False)

    perms = generic.GenericRelation(ObjectPermission,
                                          object_id_field="object_id",
                                          content_type_field="content_type")

    objects = NavManager()
    
    def __unicode__(self):
        return self.title
        
    @models.permalink
    def get_absolute_url(self):
        return('navs.detail', [self.pk])
        
    @property
    def top_items(self):
        """
        Returns all items with level 0.
        """
        return self.navitem_set.filter(level=0).order_by('ordering')
    
class NavItem(models.Model):
    nav = models.ForeignKey(Nav)
    label = models.CharField(max_length=100)
    title = models.CharField(_("Title Attribute"), max_length=100, blank=True, null=True)
    new_window = models.BooleanField(_("Open in a new window"), default=False)
    css = models.CharField(_("CSS Class"), max_length=100, blank=True, null=True)
    ordering = models.IntegerField(default=0)
    level = models.IntegerField(default=0)
    page = models.ForeignKey(Page, null=True)
    url = models.CharField(_("URL"), max_length=200, blank=True, null=True)
    
    def __unicode__(self):
        return '%s - %s' % (self.nav.title, self.label)
        
    def get_url(self):
        if self.page:
            return self.page.get_absolute_url()
        else:
            return self.url
        
    @property
    def children(self):
        """
        returns the item's direct children
        """
        #get the first sibling among the ones with greater ordering
        siblings = NavItem.objects.filter(nav=self.nav,
            ordering__gt=self.ordering,
            level=self.level).order_by('ordering')
        if siblings:
            sibling = siblings[0]
            #return all the items between the adjacent siblings.
            children = NavItem.objects.filter(nav=self.nav,
                level = self.level+1,
                ordering__gt=self.ordering,
                ordering__lt=sibling.ordering).order_by('ordering')
            return children
        else:
            # get children for the last item in a list
            children = NavItem.objects.filter(nav=self.nav,
                level = self.level+1,
                ordering__gt=self.ordering).order_by('ordering')
            return children
    
    @property
    def next(self):
        try:
            next = NavItem.objects.get(ordering=self.ordering+1, nav=self.nav)
        except NavItem.DoesNotExist:
            return None
        return next
        
    @property
    def prev(self):
        try:
            prev = NavItem.objects.get(ordering=self.ordering-1, nav=self.nav)
        except NavItem.DoesNotExist:
            return None
        return prev
    
    @property
    def next_range(self):
        if self.next:
            next = range(0, self.next.level-self.level)
        else:
            #first item
            next = range(0, self.level+1)
        return next
        
    @property
    def prev_range(self):
        if self.prev:
            prev = range(0, self.prev.level-self.level)
        else:
            #last item
            prev = range(0, self.level+1)
        return prev



import os
import shutil
import commands
from django.conf import settings
from django.core.files import File
from django.core.management.base import BaseCommand

class Command(BaseCommand):
    """
    example: python manage.py story_photos_to_files.py
    """
    def handle(self, *args, **kwargs):
        from tendenci.apps.stories.models import Story, StoryPhoto

        status, output = commands.getstatusoutput('sudo chown -R ubuntu:www-data %s' % settings.MEDIA_ROOT)
        if status > 0:
            print output
            return

        stories = Story.objects.all()
        for story in stories:
            try:
                photo = story.photo
                file = photo.file

            except (ValueError, IOError) as e:
                file = None

            if file:
                src = photo.path
                dest = photo.path.replace(settings.MEDIA_ROOT + '/stories', settings.MEDIA_ROOT + '/files/stories')
                if src != dest:
                    print "Moving %s to %s" % (src, dest)
                    dir = os.path.dirname(dest)
                    if not os.path.exists(dir):
                        os.makedirs(dir)

                    shutil.move(src, dest)
                    story.photo.save(dest.replace(settings.MEDIA_ROOT + '/files/stories/', ''), File(open(dest)))
                    story.save()
                print "Creating StoryPhoto %s for %s" % (dest, story.title)
                image = StoryPhoto(
                    creator = story.creator,
                    creator_username = story.creator_username,
                    owner = story.owner,
                    owner_username = story.owner_username)
                image.file.save(dest.replace(settings.MEDIA_ROOT + '/files/stories/', ''), File(open(dest)))
                image.save()
                if story.image:
                    old_image = story.image
                    story.image = None
                    story.save()
                    old_image.delete()
                story.image = image
                story.save()



# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):

    def forwards(self, orm):
        
        # Adding model 'Setting'
        db.create_table('site_settings_setting', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=50)),
            ('label', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('description', self.gf('django.db.models.fields.TextField')()),
            ('data_type', self.gf('django.db.models.fields.CharField')(max_length=10)),
            ('value', self.gf('django.db.models.fields.TextField')(blank=True)),
            ('default_value', self.gf('django.db.models.fields.TextField')(blank=True)),
            ('input_type', self.gf('django.db.models.fields.CharField')(max_length=25)),
            ('input_value', self.gf('django.db.models.fields.CharField')(max_length=255, blank=True)),
            ('client_editable', self.gf('django.db.models.fields.BooleanField')(default=True)),
            ('store', self.gf('django.db.models.fields.BooleanField')(default=True)),
            ('update_dt', self.gf('django.db.models.fields.DateTimeField')(auto_now=True, null=True, blank=True)),
            ('updated_by', self.gf('django.db.models.fields.CharField')(max_length=50, blank=True)),
            ('scope', self.gf('django.db.models.fields.CharField')(max_length=50)),
            ('scope_category', self.gf('django.db.models.fields.CharField')(max_length=50)),
            ('parent_id', self.gf('django.db.models.fields.IntegerField')(default=0, blank=True)),
            ('is_secure', self.gf('django.db.models.fields.BooleanField')(default=False)),
        ))
        db.send_create_signal('site_settings', ['Setting'])


    def backwards(self, orm):
        
        # Deleting model 'Setting'
        db.delete_table('site_settings_setting')


    models = {
        'site_settings.setting': {
            'Meta': {'object_name': 'Setting'},
            'client_editable': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'data_type': ('django.db.models.fields.CharField', [], {'max_length': '10'}),
            'default_value': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'description': ('django.db.models.fields.TextField', [], {}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'input_type': ('django.db.models.fields.CharField', [], {'max_length': '25'}),
            'input_value': ('django.db.models.fields.CharField', [], {'max_length': '255', 'blank': 'True'}),
            'is_secure': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'label': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'parent_id': ('django.db.models.fields.IntegerField', [], {'default': '0', 'blank': 'True'}),
            'scope': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'scope_category': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'store': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'update_dt': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'null': 'True', 'blank': 'True'}),
            'updated_by': ('django.db.models.fields.CharField', [], {'max_length': '50', 'blank': 'True'}),
            'value': ('django.db.models.fields.TextField', [], {'blank': 'True'})
        }
    }

    complete_apps = ['site_settings']



from django.template import RequestContext

from tendenci.core.base.http import Http403, render_to_403, MissingApp, render_to_missing_app


class Http403Middleware(object):
    def process_exception(self, request, exception):
        from django.contrib.auth.views import redirect_to_login
        if isinstance(exception, Http403):
            if request.user.is_anonymous():
                return redirect_to_login(request.path)
            return render_to_403(context_instance=RequestContext(request))


class MissingAppMiddleware(object):
    def process_exception(self, request, exception):
        if isinstance(exception, MissingApp):
            return render_to_missing_app(context_instance=RequestContext(request))



from django.conf.urls.defaults import *

urlpatterns = patterns('tendenci.core.payments.stripe.views',
    url(r'^payonline/(?P<payment_id>\d+)/$', 'pay_online', name="stripe.payonline"),
    url(r'^thankyou/(?P<payment_id>\d+)/$', 'thank_you', name="stripe.thank_you"),
     
)


from django.conf import settings

from tendenci.core.registry import site
from tendenci.core.registry.base import PeopleRegistry, lazy_reverse
from tendenci.addons.corporate_memberships.models import CorporateMembership


class CorporateMembershipRegistry(PeopleRegistry):
    version = '1.0'
    author = 'Schipul - The Web Marketing Company'
    author_email = 'programmers@schipul.com'
    description = 'Corporate membership management application.'
    icon = '%simages/icons/corporate-membership-color-64x64.png' % settings.STATIC_URL
    
    event_logs = {
        'corporate_membership':{
            'base':('680000','3300FF'),
            'add':('681000','3300FF'),
            'renew':('681001','471DEF'),
            'edit':('682000','1F85FF'),
            'join_approval':('682001','4D29DF'),
            'renewal_approval':('682002','5233CF'),
            'join_disapproval':('682003','563BBF'),
            'renewal_disapproval':('682004','7A6DAF'),
            'delete':('683000','B0A8CF'),
            'import':('689005','47A0BF'),
        }
    }

    url = {
        'search': lazy_reverse('corp_memb.search'),
        'list': lazy_reverse('corp_memb.list')
    }

site.register(CorporateMembership, CorporateMembershipRegistry)



from django.shortcuts import redirect
from django.contrib.auth.decorators import login_required
from django.template import RequestContext

from tendenci.core.theme.shortcuts import themed_response as render_to_response
from tendenci.core.base.http import Http403
from tendenci.core.exports.utils import run_export_task


@login_required
def export(request, template_name="boxes/export.html"):
    """Export Boxes"""

    if not request.user.is_superuser:
        raise Http403

    if request.method == 'POST':
        # initilize initial values
        fields = [
            'title',
            'content',
            'tags',
        ]
        export_id = run_export_task('boxes', 'box', fields)
        return redirect('export.status', export_id)

    return render_to_response(template_name, {
    }, context_instance=RequestContext(request))



from haystack.query import SearchQuerySet
from haystack.backends import SQ

from django.db import models
from django.db.models.query import QuerySet
from django.contrib.auth.models import Permission
from django.contrib.contenttypes.models import ContentType
from django.contrib.auth.models import User, AnonymousUser


class ObjectPermissionManager(models.Manager):
    def users_with_perms(self, perm, instance):
        """
        Checks a permission against an model instance
        and returns a list of users that have that
        permission
        """
        # check codename, return false if its a malformed codename
        try:
            codename = perm.split('.')[1]
        except IndexError:
            return []

        # TODO: create an interface that will allow users
        # to be added to the object permission table
        # this line should be uncommented after that
        return []

        # check the permissions on the object level
        content_type = ContentType.objects.get_for_model(instance)
        filters = {
            "content_type": content_type,
            "object_id": instance.pk,
            "codename": codename,
        }

        permissions = self.select_related().filter(**filters)
        users = []
        if permissions:
            # setup up list of permissions
            for perm in permissions:
                if perm.user:
                    users.append(perm.user.pk)
            return users
        else:
            return []

    def groups_with_perms(self, perm, instance):
        """
        Checks a permission against an model instance
        and returns a list of groups that have that
        permission
        """
        # check codename, return false if its a malformed codename
        try:
            codename = perm.split('.')[1]
        except IndexError:
            return []

        # check the permissions on the object level
        content_type = ContentType.objects.get_for_model(instance)
        filters = {
            "content_type": content_type,
            "object_id": instance.pk,
            "codename": codename,
        }

        permissions = self.select_related().filter(**filters)
        groups = []
        if permissions:
            # setup up list of permissions
            for perm in permissions:
                if perm.group:
                    groups.append(perm.group.pk)
            return groups
        else:
            return []

    def assign_group(self, group_or_groups, object, perms=None):
        """
        Assigns permissions to group or multiple groups
        assign_group(self, group_or_groups, object, perms=None)

        -- group_or_groups: can be a single group object, list, queryset
        or tuple. Although tuples may work differently:
        You can pass permissions individual permissions along with the tuple
        like so: ((instance,'view',),(instance,'change',))

        -- object: is the instance of a model class

        -- perms: a list of individual permissions to assign to each group
           leave blank for all permissions.
           Note: If you are using the tuple/perm approach this does nothing.
        """
        multi_group = False
        group_with_perms = False

        # nobody to give permissions too
        if not group_or_groups:
            return
        # check perms
        if not isinstance(perms, list):
            perms = None
        # check for multi_groups
        if isinstance(group_or_groups, list):
            multi_group = True
        if isinstance(group_or_groups, QuerySet):
            multi_group = True
        if isinstance(group_or_groups, tuple):
            multi_group = True
            if len(group_or_groups[0]) == 2:
                group_with_perms = True

        # treat the tuples differently. They are passed in as
        # ((group,perm,),(group,perm,) ..... (group,perm.))
        if group_with_perms:
            from tendenci.apps.user_groups.models import Group
            for group, perm in group_or_groups:
                if isinstance(group, unicode):
                    if group.isdigit():
                        try:
                            group = Group.objects.get(pk=group)
                        except:
                            group = None

                codename = '%s_%s' % (perm, object._meta.object_name.lower())
                content_type = ContentType.objects.get_for_model(object)

                perm = Permission.objects.get(codename=codename,
                                              content_type=content_type)

                defaults = {
                    "codename": codename,
                    "object_id": object.pk,
                    "content_type": perm.content_type,
                    "group": group,
                }
                self.get_or_create(**defaults)
            return  # get out

        if multi_group:
            for group in group_or_groups:
                if perms:
                    for perm in perms:
                        codename = '%s_%s' % (perm, object._meta.object_name.lower())
                        content_type = ContentType.objects.get_for_model(object)

                        perm = Permission.objects.get(codename=codename,
                                                      content_type=content_type)

                        defaults = {
                            "codename": codename,
                            "object_id": object.pk,
                            "content_type": perm.content_type,
                            "group": group,
                        }
                        self.get_or_create(**defaults)
                else:  # all default permissions
                    content_type = ContentType.objects.get_for_model(object)
                    perms = Permission.objects.filter(content_type=content_type)
                    for perm in perms:
                        defaults = {
                            "codename": perm.codename,
                            "object_id": object.pk,
                            "content_type": content_type,
                            "group": group,
                        }
                        self.get_or_create(**defaults)
        else:  # not multi_group
            if perms:
                for perm in perms:
                    codename = '%s_%s' % (perm, object._meta.object_name.lower())
                    content_type = ContentType.objects.get_for_model(object)

                    perm = Permission.objects.get(codename=codename,
                                                  content_type=content_type)
                    defaults = {
                        "codename": codename,
                        "object_id": object.pk,
                        "content_type": perm.content_type,
                        "group": group_or_groups,
                    }
                    self.get_or_create(**defaults)
            else:  # all default permissions
                content_type = ContentType.objects.get_for_model(object)
                perms = Permission.objects.filter(content_type=content_type)
                for perm in perms:
                    defaults = {
                        "codename": perm.codename,
                        "object_id": object.pk,
                        "content_type": content_type,
                        "group": group_or_groups,
                    }
                    self.get_or_create(**defaults)

    def assign(self, user_or_users, object, perms=None):
        """
        Assigns permissions to user or multiple users
        assign_group(self, user_or_users, object, perms=None)

        -- user_or_users: can be a single user object, list, queryset
        or tuple.

        -- object: is the instance of a model class

        -- perms: a list of individual permissions to assign to each user
           leave blank for all permissions.
        """
        multi_user = False

        # nobody to give permissions too
        if not user_or_users:
            return
        # check perms
        if not isinstance(perms, list):
            perms = None
        # check for multi_users
        if isinstance(user_or_users, list):
            multi_user = True
        if isinstance(user_or_users, QuerySet):
            multi_user = True
        if isinstance(user_or_users, tuple):
            multi_user = True

        if multi_user:
            for user in user_or_users:
                if perms:
                    for perm in perms:
                        codename = '%s_%s' % (perm, object._meta.object_name.lower())
                        content_type = ContentType.objects.get_for_model(object)

                        perm = Permission.objects.get(codename=codename,
                                                      content_type=content_type)

                        defaults = {
                            "codename": codename,
                            "object_id": object.pk,
                            "content_type": perm.content_type,
                            "user": user,
                        }
                        try:
                            self.get_or_create(**defaults)
                        except self.model.MultipleObjectsReturned as e:
                            pass

                else:  # all default permissions
                    content_type = ContentType.objects.get_for_model(object)
                    perms = Permission.objects.filter(content_type=content_type)
                    for perm in perms:
                        defaults = {
                            "codename": perm.codename,
                            "object_id": object.pk,
                            "content_type": content_type,
                            "user": user,
                        }
                        try:
                            self.get_or_create(**defaults)
                        except self.model.MultipleObjectsReturned as e:
                            pass

        else:  # not muli_user
            if perms:
                for perm in perms:
                    codename = '%s_%s' % (perm, object._meta.object_name.lower())
                    content_type = ContentType.objects.get_for_model(object)

                    perm = Permission.objects.get(codename=codename,
                                                  content_type=content_type)
                    defaults = {
                        "codename": codename,
                        "object_id": object.pk,
                        "content_type": perm.content_type,
                        "user": user_or_users,
                    }
                    try:
                        self.get_or_create(**defaults)
                    except self.model.MultipleObjectsReturned as e:
                        pass

            else:  # all default permissions
                content_type = ContentType.objects.get_for_model(object)
                perms = Permission.objects.filter(content_type=content_type)
                for perm in perms:
                    defaults = {
                        "codename": perm.codename,
                        "object_id": object.pk,
                        "content_type": content_type,
                        "user": user_or_users,
                    }
                    try:
                        self.get_or_create(**defaults)
                    except self.model.MultipleObjectsReturned as e:
                        pass

    def remove_all(self, object):
        """
            Remove all permissions on object (instance)
        """
        content_type = ContentType.objects.get_for_model(object)
        perms = self.filter(content_type=content_type,
                            object_id=object.pk)
        for perm in perms:
            perm.delete()


class TendenciBaseManager(models.Manager):
    """
    Base manager for all TendenciBase models
    """
    user = None

    # Private functions
    def _anon_sqs(self, sqs, **kwargs):
        """
        Filter the query set for anonymous users
        """
        status_detail = kwargs.get('status_detail', 'active')
        status = kwargs.get('status', True)

        sqs = sqs.filter(
            allow_anonymous_view=True,
            status=status,
            status_detail=status_detail)
        return sqs

    def _member_sqs(self, sqs, user, **kwargs):
        """
        Filter the query set for members.
        
        (status AND status_detail AND
        (anon_view OR user_view OR member_view))
        OR
        (users_can_view__in user.pk)
        OR
        (groups_can_view__in user's groups)
        
        user is a required argument since we'll be filtering by user.pk.
        """
        groups = [g.pk for g in user.group_set.all()]
        status_detail = kwargs.get('status_detail', 'active')
        status = kwargs.get('status', True)

        anon_q = SQ(allow_anonymous_view=True)
        user_q = SQ(allow_user_view=True)
        member_q = SQ(allow_member_view=True)
        status_q = SQ(status=status, status_detail=status_detail)
        user_perm_q = SQ(users_can_view__in=[user.pk])
        group_perm_q = SQ(groups_can_view__in=groups)
        
        creator_q = SQ(creator_username=user.username)
        owner_q = SQ(owner_username=user.username)
        
        if groups:
            sqs = sqs.filter(
                (status_q & (anon_q | user_q | creator_q | owner_q)) |
                (user_perm_q | group_perm_q))
        else:
            sqs = sqs.filter(
                (status_q & (anon_q | user_q | creator_q | owner_q)) |
                (user_perm_q))

        return sqs

    def _user_sqs(self, sqs, user, **kwargs):
        """
        Filter the query set for people between admin and anon permission

        (status AND status_detail AND ((anon_view OR user_view)
        OR
        (users_can_view__in user.pk)
        OR
        (groups_can_view__in user's groups)

        user required since we'll filter by user.pk.
        """
        groups = [g.pk for g in user.group_set.all()]
        status_detail = kwargs.get('status_detail', 'active')
        status = kwargs.get('status', True)

        anon_q = SQ(allow_anonymous_view=True)
        user_q = SQ(allow_user_view=True)
        status_q = SQ(status=status, status_detail=status_detail)
        user_perm_q = SQ(users_can_view__in=[user.pk])
        group_perm_q = SQ(groups_can_view__in=groups)

        if groups:
            sqs = sqs.filter(
                (status_q & (anon_q | user_q)) |
                (user_perm_q | group_perm_q))
        else:
            sqs = sqs.filter(
                (status_q & (anon_q | user_q)) |
                (user_perm_q)
            )

        return sqs
    
    def _user_or_member_sqs(self, sqs, user, **kwargs):
        """
        Filter the query set for user or members.
        
        Here we check the existence of the fields to be tested, 
        and handle the queries that need to be directly pulled from db.

        (status AND status_detail AND
        (anon_view OR user_view OR member_view))
        OR
        (users_can_view__in user.pk)
        OR
        (groups_can_view__in user's groups)

        user is a required argument since we'll be filtering by user.pk.
        """
        groups = [g.pk for g in user.group_set.all()]
        status_detail = kwargs.get('status_detail', 'active')
        status = kwargs.get('status', True)
        is_member = kwargs.get('is_member', True)
        
        # allow_xxx and creator and owner fields to be checked
        fields_ors_sq = None
        fields_ors_sq_list = []
        model_fiels =[field.name for field in self.model._meta.fields]

        if 'allow_anonymous_view' in model_fiels:
            fields_ors_sq_list.append(SQ(allow_anonymous_view=True))
        if 'allow_user_view' in model_fiels:
            fields_ors_sq_list.append(SQ(allow_user_view=True))
        if is_member:
            if 'allow_member_view' in model_fiels:
                fields_ors_sq_list.append(SQ(allow_member_view=True))
        if 'creator_username' in model_fiels:
            fields_ors_sq_list.append(SQ(creator_username=user.username))
        if 'owner_username' in model_fiels:
            fields_ors_sq_list.append(SQ(owner_username=user.username))
            
        for field_sq in fields_ors_sq_list:
            if fields_ors_sq:
                fields_ors_sq = fields_ors_sq | field_sq
            else:
                fields_ors_sq = field_sq
        
        # status and status_detail if exist                
        status_d = {}
        if 'status' in model_fiels:
            status_d.update({'status': status})
        if 'status_detail' in model_fiels:
            status_d.update({'status_detail': status_detail})
            
        if status_d:
            status_q = SQ(**status_d)
        else:
            status_q = None
           
        direct_db = kwargs.get('direct_db', 0) 
        
        # object permissions
        
        if not direct_db:
            group_perm_q = SQ(groups_can_view__in=groups)
            # user is not being recorded in the ObjectPermission
            #user_perm_q = SQ(users_can_view__in=[user.pk])
        else:
            # to pull directly from db, get a list of object_ids instead of groups
            from tendenci.core.perms.object_perms import ObjectPermission

            app_label = self.model._meta.app_label
            model_name = self.model.__name__
            content_type = ContentType.objects.filter(
                                        app_label=app_label,
                                        model=model_name)[:1]
            codename = 'view_%s' % model_name
            group_allowed_object_ids = ObjectPermission.objects.filter(
                                            content_type=content_type[0],
                                            codename=codename,
                                            group__in=groups).values_list('id', flat=True)
            group_perm_q = SQ(id__in=group_allowed_object_ids)
            
            # user is not being recorded in the ObjectPermission
            # so, commenting it out for now
#            user_allowed_object_ids = ObjectPermission.objects.filter(
#                                            content_type=content_type[0],
#                                            codename=codename,
#                                            user__in=[user]).values_list('id', flat=True)
            group_perm_q = SQ(id__in=group_allowed_object_ids)
            
            
        filters = None
        if status_q or fields_ors_sq:
            if status_q and fields_ors_sq:
                filters = status_q & fields_ors_sq
            elif status_q:
                filters = status_q
            else:
                filters = fields_ors_sq
        
        if groups:
            if not filters:
                filters = group_perm_q
            else:
                filters = filters | group_perm_q
                
        if filters:
            sqs = sqs.filter(filters)
        
        return sqs

    def _impersonation(self, user):
        """
        Test for impersonation and return the impersonee
        """
        if hasattr(user, 'impersonated_user'):
            if isinstance(user.impersonated_user, User):
                user = user.impersonated_user
        return user

    def _permissions_sqs(self, sqs, user, status, status_detail, **kwargs):
        if user.is_authenticated() and user.profile.is_superuser:
            sqs = sqs.all()
        else:
            if user.is_anonymous():
                sqs = self._anon_sqs(sqs, status=status, status_detail=status_detail)
            elif user.profile.is_member:
                sqs = self._member_sqs(sqs, user, status=status,
                    status_detail=status_detail)
            else:
                sqs = self._user_sqs(sqs, user, status=status,
                    status_detail=status_detail)
           
        return sqs

    # Public functions
    def search(self, query=None, *args, **kwargs):
        """
        Search the Django Haystack search index
        Returns a SearchQuerySet object
        """
        sqs = kwargs.get('sqs', SearchQuerySet())
        direct_db = kwargs.get('direct_db', 0)

        # filter out the big parts first
        if not direct_db:
            sqs = sqs.models(self.model)

        # user information
        user = kwargs.get('user') or AnonymousUser()
        user = self._impersonation(user)
        self.user = user

        # if the status_detail is something like "published"
        # then you can specify the kwargs to override
        status_detail = kwargs.get('status_detail', 'active')
        status = kwargs.get('status', True)

        if query:
            sqs = sqs.auto_query(sqs.query.clean(query))

        sqs = self._permissions_sqs(sqs, user, status, status_detail, direct_db=direct_db)

        return sqs



from django.core.management.base import BaseCommand


class Command(BaseCommand):
    """ Command to cache the site_url dicts for use in the sitemap.xml """

    def handle(self, *args, **options):
        from django.core.cache import cache
        from django.conf import settings
        from django.contrib.sites.models import Site
        from tendenci.core.sitemaps.views import get_all_sitemaps

        sitemaps = dict([(cls.__name__, cls) for cls in get_all_sitemaps()])
        maps = sitemaps.values()
        for site in maps:
            if callable(site):
                site = site()
                site_key = site.__class__.__name__
            else:
                site_key = site.__name__
            sitemap_cache_key = '.'.join([settings.SITE_CACHE_KEY, 'sitemap_cache', site_key])
            site_urls = cache.get(sitemap_cache_key)
            if not isinstance(site_urls, list):
                site_urls = site.get_urls(site=Site.objects.get_current())
                cache.set(sitemap_cache_key, list(site_urls), 86400)
                print "Caching %s" % site_key

        print "Sitemap is cached."



from django.conf.urls.defaults import patterns, url, include

urlpatterns = patterns('tendenci.core.payments.views',
    url(r'^payonline/(?P<invoice_id>\d+)/(?P<guid>[\d\w-]+)?$', 'pay_online', name="payment.pay_online"),
    (r'^authorizenet/', include('tendenci.core.payments.authorizenet.urls')),
    (r'^firstdata/', include('tendenci.core.payments.firstdata.urls')),
    (r'^payflowlink/', include('tendenci.core.payments.payflowlink.urls')),
    (r'^paypal/', include('tendenci.core.payments.paypal.urls')),
    (r'^stripe/', include('tendenci.core.payments.stripe.urls')),
    url(r'^(?P<id>\d+)/(?P<guid>[\d\w-]+)?$', 'view', name="payment.view"), 
    url(r'^receipt/(?P<id>\d+)/(?P<guid>[\d\w-]+)$', 'receipt', name="payment.receipt"), 
)


"""
Base backends structures.

This module defines base classes needed to define custom OpenID or OAuth
auth services from third parties. This customs must subclass an Auth and
and Backend class, check current implementation for examples.

Also the modules *must* define a BACKENDS dictionary with the backend name
(which is used for URLs matching) and Auth class, otherwise it won't be
enabled.
"""
from os import walk
from os.path import basename
from uuid import uuid4
from urllib2 import Request, urlopen
from urllib import urlencode
from httplib import HTTPSConnection

from openid.consumer.consumer import Consumer, SUCCESS, CANCEL, FAILURE
from openid.consumer.discover import DiscoveryFailure
from openid.extensions import sreg, ax

from oauth2 import Consumer as OAuthConsumer, Token, Request as OAuthRequest, \
                   SignatureMethod_HMAC_SHA1

from django.conf import settings
from django.core.exceptions import MultipleObjectsReturned
from django.contrib.auth import authenticate
from django.contrib.auth.backends import ModelBackend
from django.utils import simplejson
from django.utils.importlib import import_module

from tendenci.addons.social_auth.models import UserSocialAuth
from tendenci.addons.social_auth.store import DjangoOpenIDStore
from tendenci.addons.social_auth.signals import pre_update, socialauth_registered

from tendenci.core.site_settings.utils import get_setting
from tendenci.core.event_logs.models import EventLog

# OpenID configuration
OLD_AX_ATTRS = [
    ('http://schema.openid.net/contact/email', 'old_email'),
    ('http://schema.openid.net/namePerson', 'old_fullname'),
    ('http://schema.openid.net/namePerson/friendly', 'old_nickname')
]
AX_SCHEMA_ATTRS = [
    # Request both the full name and first/last components since some
    # providers offer one but not the other.
    ('http://axschema.org/contact/email', 'email'),
    ('http://axschema.org/namePerson', 'fullname'),
    ('http://axschema.org/namePerson/first', 'first_name'),
    ('http://axschema.org/namePerson/last', 'last_name'),
    ('http://axschema.org/namePerson/friendly', 'nickname'),
]
SREG_ATTR = [
    ('email', 'email'),
    ('fullname', 'fullname'),
    ('nickname', 'nickname')
]
OPENID_ID_FIELD = 'openid_identifier'
SESSION_NAME = 'openid'

# key for username in user details dict used around, see get_user_details
# method
USERNAME = 'username'
# get User class, could not be auth.User
User = UserSocialAuth._meta.get_field('user').rel.to
# username field max length
USERNAME_MAX_LENGTH = User._meta.get_field(USERNAME).max_length
# uuid hex characters to keep while generating unique usernames
UUID_MAX_LENGTH = 16

# a few settings values
def _setting(name, default=None):
    return getattr(settings, name, default)

CREATE_USERS = _setting('SOCIAL_AUTH_CREATE_USERS', True)
ASSOCIATE_BY_MAIL = _setting('SOCIAL_AUTH_ASSOCIATE_BY_MAIL', False)
LOAD_EXTRA_DATA = _setting('SOCIAL_AUTH_EXTRA_DATA', True)
FORCE_RANDOM_USERNAME = _setting('SOCIAL_AUTH_FORCE_RANDOM_USERNAME', False)
USERNAME_FIXER = _setting('SOCIAL_AUTH_USERNAME_FIXER', lambda u: u)
DEFAULT_USERNAME = _setting('SOCIAL_AUTH_DEFAULT_USERNAME')
CHANGE_SIGNAL_ONLY = _setting('SOCIAL_AUTH_CHANGE_SIGNAL_ONLY', False)


class SocialAuthBackend(ModelBackend):
    """A django.contrib.auth backend that authenticates the user based on
    a authentication provider response"""
    name = ''  # provider name, it's stored in database

    def authenticate(self, *args, **kwargs):
        """Authenticate user using social credentials

        Authentication is made if this is the correct backend, backend
        verification is made by kwargs inspection for current backend
        name presence.
        """
        # Validate backend and arguments. Require that the Social Auth
        # response be passed in as a keyword argument, to make sure we
        # don't match the username/password calling conventions of
        # authenticate.
        if not (self.name and kwargs.get(self.name) and 'response' in kwargs):
            return None

        response = kwargs.get('response')
        request = kwargs.get('request')
        details = self.get_user_details(response)
        uid = self.get_user_id(details, response)
        is_new = False
        try:
            social_user = UserSocialAuth.objects.select_related('user')\
                                                .get(provider=self.name,
                                                     uid=uid)
        except UserSocialAuth.DoesNotExist:
            user = kwargs.get('user')
            if user is None:  # new user
                if not CREATE_USERS:
                    return None

                email = details.get('email')
                if email and ASSOCIATE_BY_MAIL:
                    # try to associate accounts registered with the same email
                    # address, only if it's a single object. ValueError is
                    # raised if multiple objects are returned
                    try:
                        user = User.objects.get(email=email)
                    except MultipleObjectsReturned:
                        raise ValueError('Not unique email address supplied')
                    except User.DoesNotExist:
                        user = None
                if not user:
                    username = self.username(details)
                    user = User.objects.create_user(username=username,
                                                    email=email)
                    is_new = True
                    default_user_groups =[g.strip() for g in (get_setting('module', 'users', 'defaultusergroup')).split(',')]
                    if default_user_groups:
                        from tendenci.apps.user_groups.models import Group, GroupMembership
                        from django.db.models import Q
                        for group_name in default_user_groups:
                            groups = Group.objects.filter(Q(name=group_name) | Q(label=group_name)).filter(allow_self_add=1, status=1, status_detail='active')
                            if groups:
                                group = groups[0]
                            else:
                                # group doesnot exist, so create the group
                                group = Group()
                                group.name  = group_name
                                group.label = group_name
                                group.type = 'distribution'
                                group.show_as_option = 1
                                group.allow_self_add = 1
                                group.allow_self_remove = 1
                                group.creator = user
                                group.creator_username = user.username
                                group.owner =  user
                                group.owner_username = user.username
                                try:
                                    group.save()
                                except:
                                    group = None
                                
                            if group:
                                gm = GroupMembership()
                                gm.group = group
                                gm.member = user
                                gm.creator_id = user.id
                                gm.creator_username = user.username
                                gm.owner_id =  user.id
                                gm.owner_username = user.username
                                gm.save()
                    log_defaults = {
                        'event_id' : 121000,
                        'event_data': '%s (%d) self added by form' % (user._meta.object_name, user.pk),
                        'description': '%s self added' % user._meta.object_name,
                        'user': user,
                        'request': request,
                        'instance': user,
                    }
                    EventLog.objects.log(**log_defaults)
            social_user = self.associate_auth(user, uid, response, details)
        else:
            # This account was registered to another user, so we raise an
            # error in such case and the view should decide what to do on
            # at this moment, merging account is not an option because that
            # would imply update user references on other apps, that's too
            # much intrusive
            if 'user' in kwargs and kwargs['user'] != social_user.user:
                raise ValueError('Account already in use.', social_user)
            user = social_user.user

        # Update user account data.
        self.update_user_details(user, response, details, is_new)

        # Update extra_data storage, unless disabled by setting
        if LOAD_EXTRA_DATA:
            extra_data = self.extra_data(user, uid, response, details)
            if extra_data and social_user.extra_data != extra_data:
                social_user.extra_data = extra_data
                social_user.save()

        return user

    def username(self, details):
        """Return an unique username, if SOCIAL_AUTH_FORCE_RANDOM_USERNAME
        setting is True, then username will be a random USERNAME_MAX_LENGTH
        chars uuid generated hash
        """
        def get_random_username():
            """Return hash from unique string cut at username max length"""
            return uuid4().get_hex()[:USERNAME_MAX_LENGTH]

        if FORCE_RANDOM_USERNAME:
            username = get_random_username()
        elif USERNAME in details:
            username = details[USERNAME]
        elif DEFAULT_USERNAME:
            username = DEFAULT_USERNAME
            if callable(username):
                username = username()
        else:
            username = None
        username = username or get_random_username()

        name = username
        final_username = None

        while not final_username:
            try:
                fixed_name = USERNAME_FIXER(name)
                User.objects.get(username=fixed_name)
            except User.DoesNotExist:
                final_username = fixed_name
            else:
                # User with same username already exists, generate a unique
                # username for current user using username as base but adding
                # a unique hash at the end.
                #
                # Generate an uuid number but keep only the needed to avoid
                # breaking the field max_length value, this reduces the
                # uniqueness, but it's less likely to happen repetitions than
                # increasing an index.
                if len(username) + UUID_MAX_LENGTH > USERNAME_MAX_LENGTH:
                    username = username[:USERNAME_MAX_LENGTH - UUID_MAX_LENGTH]
                name = username + uuid4().get_hex()[:UUID_MAX_LENGTH]

        return final_username

    def associate_auth(self, user, uid, response, details):
        """Associate a Social Auth with an user account."""
        return UserSocialAuth.objects.create(user=user, uid=uid,
                                             provider=self.name)

    def extra_data(self, user, uid, response, details):
        """Return default blank user extra data"""
        return ''

    def update_user_details(self, user, response, details, is_new=False):
        """Update user details with (maybe) new data. Username is not
        changed if associating a new credential."""
        changed = False  # flag to track changes

        # check if values update should be left to signals handlers only
        if not CHANGE_SIGNAL_ONLY:
            for name, value in details.iteritems():
                # do not update username, it was already generated by
                # self.username(...) and loaded in given instance
                if name != USERNAME and value and value != getattr(user, name,
                                                                   value):
                    setattr(user, name, value)
                    changed = True

        # Fire a pre-update signal sending current backend instance,
        # user instance (created or retrieved from database), service
        # response and processed details.
        #
        # Also fire socialauth_registered signal for newly registered
        # users.
        #
        # Signal handlers must return True or False to signal instance
        # changes. Send method returns a list of tuples with receiver
        # and it's response.
        signal_response = lambda (receiver, response): response

        kwargs = {'sender': self.__class__, 'user': user,
                  'response': response, 'details': details}
        changed |= any(filter(signal_response, pre_update.send(**kwargs)))

        # Fire socialauth_registered signal on new user registration
        if is_new:
            changed |= any(filter(signal_response,
                                  socialauth_registered.send(**kwargs)))

        if changed:
            user.save()

    def get_user_id(self, details, response):
        """Must return a unique ID from values returned on details"""
        raise NotImplementedError('Implement in subclass')

    def get_user_details(self, response):
        """Must return user details in a know internal struct:
            {USERNAME: <username if any>,
             'email': <user email if any>,
             'fullname': <user full name if any>,
             'first_name': <user first name if any>,
             'last_name': <user last name if any>}
        """
        raise NotImplementedError('Implement in subclass')

    def get_user(self, user_id):
        """Return user instance for @user_id"""
        try:
            return User.objects.get(pk=user_id)
        except User.DoesNotExist:
            return None


class OAuthBackend(SocialAuthBackend):
    """OAuth authentication backend base class.

    EXTRA_DATA defines a set of name that will be stored in
               extra_data field. It must be a list of tuples with
               name and alias.

    Also settings will be inspected to get more values names that should be
    stored on extra_data field. Setting name is created from current backend
    name (all uppercase) plus _EXTRA_DATA.

    access_token is always stored.
    """
    EXTRA_DATA = None

    def get_user_id(self, details, response):
        "OAuth providers return an unique user id in response"""
        return response['id']

    def extra_data(self, user, uid, response, details):
        """Return access_token and extra defined names to store in
        extra_data field"""
        data = {'access_token': response.get('access_token', '')}
        name = self.name.replace('-', '_').upper()
        names = (self.EXTRA_DATA or []) + _setting(name + '_EXTRA_DATA', [])
        data.update((alias, response.get(name)) for name, alias in names)
        return data


class OpenIDBackend(SocialAuthBackend):
    """Generic OpenID authentication backend"""
    name = 'openid'

    def get_user_id(self, details, response):
        """Return user unique id provided by service"""
        return response.identity_url

    def values_from_response(self, response, sreg_names=None, ax_names=None):
        """Return values from SimpleRegistration response or
        AttributeExchange response if present.

        @sreg_names and @ax_names must be a list of name and aliases
        for such name. The alias will be used as mapping key.
        """
        values = {}

        # Use Simple Registration attributes if provided
        if sreg_names:
            resp = sreg.SRegResponse.fromSuccessResponse(response)
            if resp:
                values.update((alias, resp.get(name) or '')
                                    for name, alias in sreg_names)

        # Use Attribute Exchange attributes if provided
        if ax_names:
            resp = ax.FetchResponse.fromSuccessResponse(response)
            if resp:
                values.update((alias.replace('old_', ''),
                               resp.getSingle(src, ''))
                                for src, alias in ax_names)
        return values

    def get_user_details(self, response):
        """Return user details from an OpenID request"""
        values = {USERNAME: '', 'email': '', 'fullname': '',
                  'first_name': '', 'last_name': ''}
        # update values using SimpleRegistration or AttributeExchange
        # values
        values.update(self.values_from_response(response,
                                                SREG_ATTR,
                                                OLD_AX_ATTRS + \
                                                AX_SCHEMA_ATTRS))

        fullname = values.get('fullname') or ''
        first_name = values.get('first_name') or ''
        last_name = values.get('last_name') or ''

        if not fullname and first_name and last_name:
            fullname = first_name + ' ' + last_name
        elif fullname:
            try:  # Try to split name for django user storage
                first_name, last_name = fullname.rsplit(' ', 1)
            except ValueError:
                last_name = fullname

        values.update({'fullname': fullname, 'first_name': first_name,
                       'last_name': last_name,
                       USERNAME: values.get(USERNAME) or \
                                   (first_name.title() + last_name.title())})
        return values

    def extra_data(self, user, uid, response, details):
        """Return defined extra data names to store in extra_data field.
        Settings will be inspected to get more values names that should be
        stored on extra_data field. Setting name is created from current
        backend name (all uppercase) plus _SREG_EXTRA_DATA and
        _AX_EXTRA_DATA because values can be returned by SimpleRegistration
        or AttributeExchange schemas.

        Both list must be a value name and an alias mapping similar to
        SREG_ATTR, OLD_AX_ATTRS or AX_SCHEMA_ATTRS
        """
        name = self.name.replace('-', '_').upper()
        sreg_names = _setting(name + '_SREG_EXTRA_DATA')
        ax_names = _setting(name + '_AX_EXTRA_DATA')
        data = self.values_from_response(response, ax_names, sreg_names)
        return data


class BaseAuth(object):
    """Base authentication class, new authenticators should subclass
    and implement needed methods.

        @AUTH_BACKEND   Authorization backend related with this service
    """
    AUTH_BACKEND = None

    def __init__(self, request, redirect):
        self.request = request
        # Use request because some auth providers use POST urls with needed
        # GET parameters on it
        self.data = request.REQUEST
        self.redirect = redirect

    def auth_url(self):
        """Must return redirect URL to auth provider"""
        raise NotImplementedError('Implement in subclass')

    def auth_html(self):
        """Must return login HTML content returned by provider"""
        raise NotImplementedError('Implement in subclass')

    def auth_complete(self, *args, **kwargs):
        """Completes loging process, must return user instance"""
        raise NotImplementedError('Implement in subclass')

    @property
    def uses_redirect(self):
        """Return True if this provider uses redirect url method,
        otherwise return false."""
        return True

    @classmethod
    def enabled(cls):
        """Return backend enabled status, all enabled by default"""
        return True

    def disconnect(self, user):
        """Deletes current backend from user if associated.
        Override if extra operations are needed.
        """
        user.social_auth.filter(provider=self.AUTH_BACKEND.name).delete()


class OpenIdAuth(BaseAuth):
    """OpenId process handling"""
    AUTH_BACKEND = OpenIDBackend

    def auth_url(self):
        """Return auth URL returned by service"""
        openid_request = self.setup_request()
        # Construct completion URL, including page we should redirect to
        return_to = self.request.build_absolute_uri(self.redirect)
        return openid_request.redirectURL(self.trust_root(), return_to)

    def auth_html(self):
        """Return auth HTML returned by service"""
        openid_request = self.setup_request()
        return_to = self.request.build_absolute_uri(self.redirect)
        form_tag = {'id': 'openid_message'}
        return openid_request.htmlMarkup(self.trust_root(), return_to,
                                         form_tag_attrs=form_tag)

    def trust_root(self):
        """Return trust-root option"""
        return _setting('OPENID_TRUST_ROOT',
                        self.request.build_absolute_uri('/'))

    def auth_complete(self, *args, **kwargs):
        """Complete auth process"""
        response = self.consumer().complete(dict(self.data.items()),
                                            self.request.build_absolute_uri())
        if not response:
            raise ValueError('This is an OpenID relying party endpoint')
        elif response.status == SUCCESS:
            kwargs.update({'response': response, self.AUTH_BACKEND.name: True})
            return authenticate(*args, **kwargs)
        elif response.status == FAILURE:
            raise ValueError('OpenID authentication failed: %s' % \
                             response.message)
        elif response.status == CANCEL:
            raise ValueError('Authentication cancelled')
        else:
            raise ValueError('Unknown OpenID response type: %r' % \
                             response.status)

    def setup_request(self):
        """Setup request"""
        openid_request = self.openid_request()
        # Request some user details. Use attribute exchange if provider
        # advertises support.
        if openid_request.endpoint.supportsType(ax.AXMessage.ns_uri):
            fetch_request = ax.FetchRequest()
            # Mark all attributes as required, Google ignores optional ones
            for attr, alias in (AX_SCHEMA_ATTRS + OLD_AX_ATTRS):
                fetch_request.add(ax.AttrInfo(attr, alias=alias,
                                              required=True))
        else:
            fetch_request = sreg.SRegRequest(optional=dict(SREG_ATTR).keys())
        openid_request.addExtension(fetch_request)

        return openid_request

    def consumer(self):
        """Create an OpenID Consumer object for the given Django request."""
        return Consumer(self.request.session.setdefault(SESSION_NAME, {}),
                        DjangoOpenIDStore())

    @property
    def uses_redirect(self):
        """Return true if openid request will be handled with redirect or
        HTML content will be returned.
        """
        if not hasattr(self, '_uses_redirect'):
            setattr(self, '_uses_redirect',
                    self.openid_request().shouldSendRedirect())
        return getattr(self, '_uses_redirect', True)

    def openid_request(self):
        """Return openid request"""
        if not hasattr(self, '_openid_request'):
            openid_url = self.openid_url()
            try:
                openid_request = self.consumer().begin(openid_url)
            except DiscoveryFailure, err:
                raise ValueError('OpenID discovery error: %s' % err)
            else:
                setattr(self, '_openid_request', openid_request)
        return getattr(self, '_openid_request', None)

    def openid_url(self):
        """Return service provider URL.
        This base class is generic accepting a POST parameter that specifies
        provider URL."""
        if OPENID_ID_FIELD not in self.data:
            raise ValueError('Missing openid identifier')
        return self.data[OPENID_ID_FIELD]


class BaseOAuth(BaseAuth):
    """OAuth base class"""
    def __init__(self, request, redirect):
        """Init method"""
        super(BaseOAuth, self).__init__(request, redirect)
        self.redirect_uri = self.request.build_absolute_uri(self.redirect)


class ConsumerBasedOAuth(BaseOAuth):
    """Consumer based mechanism OAuth authentication, fill the needed
    parameters to communicate properly with authentication service.

        @AUTHORIZATION_URL       Authorization service url
        @REQUEST_TOKEN_URL       Request token URL
        @ACCESS_TOKEN_URL        Access token URL
        @SERVER_URL              Authorization server URL
    """
    AUTHORIZATION_URL = ''
    REQUEST_TOKEN_URL = ''
    ACCESS_TOKEN_URL = ''
    SERVER_URL = ''
    SETTINGS_KEY_NAME = ''
    SETTINGS_SECRET_NAME = ''

    def auth_url(self):
        """Return redirect url"""
        token = self.unauthorized_token()
        name = self.AUTH_BACKEND.name + 'unauthorized_token_name'
        self.request.session[name] = token.to_string()
        return self.oauth_request(token, self.AUTHORIZATION_URL).to_url()

    def auth_complete(self, *args, **kwargs):
        """Return user, might be logged in"""
        name = self.AUTH_BACKEND.name + 'unauthorized_token_name'
        unauthed_token = self.request.session.get(name)
        if not unauthed_token:
            raise ValueError('Missing unauthorized token')

        token = Token.from_string(unauthed_token)
        if token.key != self.data.get('oauth_token', 'no-token'):
            raise ValueError('Incorrect tokens')

        access_token = self.access_token(token)
        data = self.user_data(access_token)
        if data is not None:
            data['access_token'] = access_token.to_string()

        kwargs.update({'response': data, self.AUTH_BACKEND.name: True})
        return authenticate(*args, **kwargs)

    def unauthorized_token(self):
        """Return request for unauthorized token (first stage)"""
        request = self.oauth_request(token=None, url=self.REQUEST_TOKEN_URL)
        response = self.fetch_response(request)
        return Token.from_string(response)

    def oauth_request(self, token, url, extra_params=None):
        """Generate OAuth request, setups callback url"""
        params = {'oauth_callback': self.redirect_uri}
        if extra_params:
            params.update(extra_params)

        if 'oauth_verifier' in self.data:
            params['oauth_verifier'] = self.data['oauth_verifier']
        request = OAuthRequest.from_consumer_and_token(self.consumer,
                                                       token=token,
                                                       http_url=url,
                                                       parameters=params)
        request.sign_request(SignatureMethod_HMAC_SHA1(), self.consumer, token)
        return request

    def fetch_response(self, request):
        """Executes request and fetchs service response"""
        connection = HTTPSConnection(self.SERVER_URL)
        connection.request(request.method.upper(), request.to_url())
        return connection.getresponse().read()

    def access_token(self, token):
        """Return request for access token value"""
        request = self.oauth_request(token, self.ACCESS_TOKEN_URL)
        return Token.from_string(self.fetch_response(request))

    def user_data(self, access_token):
        """Loads user data from service"""
        raise NotImplementedError('Implement in subclass')

    @property
    def consumer(self):
        """Setups consumer"""
        consumer = getattr(self, '_consumer', None)
        if consumer is None:
            consumer = OAuthConsumer(*self.get_key_and_secret())
            setattr(self, '_consumer', consumer)
        return consumer

    def get_key_and_secret(self):
        """Return tuple with Consumer Key and Consumer Secret for current
        service provider. Must return (key, secret), order *must* be respected.
        """
        return _setting(self.SETTINGS_KEY_NAME), \
               _setting(self.SETTINGS_SECRET_NAME)

    @classmethod
    def enabled(cls):
        """Return backend enabled status by checking basic settings"""
        return all(hasattr(settings, name) for name in
                        (cls.SETTINGS_KEY_NAME, cls.SETTINGS_SECRET_NAME))


class BaseOAuth2(BaseOAuth):
    """Base class for OAuth2 providers.

    OAuth2 draft details at:
        http://tools.ietf.org/html/draft-ietf-oauth-v2-10

    Attributes:
        @AUTHORIZATION_URL       Authorization service url
        @ACCESS_TOKEN_URL        Token URL
    """
    AUTHORIZATION_URL = None
    ACCESS_TOKEN_URL = None

    def auth_url(self):
        """Return redirect url"""
        client_id, client_secret = self.get_key_and_secret()
        args = {'client_id': client_id,
                'scope': ' '.join(self.get_scope()),
                'redirect_uri': self.redirect_uri,
                'response_type': 'code'}  # requesting code
        return self.AUTHORIZATION_URL + '?' + urlencode(args)

    def auth_complete(self, *args, **kwargs):
        """Completes loging process, must return user instance"""
        client_id, client_secret = self.get_key_and_secret()
        params = {'grant_type': 'authorization_code',  # request auth code
                  'code': self.data.get('code', ''),  # server response code
                  'client_id': client_id,
                  'client_secret': client_secret,
                  'redirect_uri': self.redirect_uri}
        headers = {'Content-Type': 'application/x-www-form-urlencoded'}
        request = Request(self.ACCESS_TOKEN_URL, data=urlencode(params),
                          headers=headers)

        try:
            response = simplejson.loads(urlopen(request).read())
        except (ValueError, KeyError):
            raise ValueError('Unknown OAuth2 response type')

        if response.get('error'):
            error = response.get('error_description') or response.get('error')
            raise ValueError('OAuth2 authentication failed: %s' % error)
        else:
            response.update(self.user_data(response['access_token']) or {})
            kwargs.update({'response': response, self.AUTH_BACKEND.name: True})
            return authenticate(*args, **kwargs)

    def get_scope(self):
        """Return list with needed access scope"""
        return []

    def get_key_and_secret(self):
        """Return tuple with Consumer Key and Consumer Secret for current
        service provider. Must return (key, secret), order *must* be respected.
        """
        return _setting(self.SETTINGS_KEY_NAME), \
               _setting(self.SETTINGS_SECRET_NAME)


# import sources from where check for auth backends
SOCIAL_AUTH_IMPORT_SOURCES = (
    'social_auth.backends',
    'social_auth.backends.contrib',
) + _setting('SOCIAL_AUTH_IMPORT_BACKENDS', ())

def get_backends():
    backends = {}

    for mod_name in SOCIAL_AUTH_IMPORT_SOURCES:
        try:
            mod = import_module(mod_name)
        except ImportError:
            continue

        for directory, subdir, files in walk(mod.__path__[0]):
            for name in filter(lambda name: name.endswith('.py'), files):
                try:
                    name = basename(name).replace('.py', '')
                    sub = import_module(mod_name + '.' + name)
                    # register only enabled backends
                    backends.update(((key, val)
                                        for key, val in sub.BACKENDS.items()
                                            if val.enabled()))
                except (ImportError, AttributeError):
                    pass
    return backends

# load backends from defined modules
BACKENDS = get_backends()
BACKENDS[OpenIdAuth.AUTH_BACKEND.name] = OpenIdAuth

def get_backend(name, *args, **kwargs):
    """Return auth backend instance *if* it's registered, None in other case"""
    return BACKENDS.get(name, lambda *args, **kwargs: None)(*args, **kwargs)



from haystack import indexes, site

from django.utils.html import strip_tags, strip_entities

from tendenci.addons.memberships.models import App, AppEntry, Membership
from tendenci.core.perms.indexes import TendenciBaseSearchIndex


class MembershipIndex(TendenciBaseSearchIndex):
    corporate_membership_id = indexes.IntegerField(model_attr='corporate_membership_id')
    member_number = indexes.CharField(model_attr='member_number')
    mem_type = indexes.IntegerField()
    first_name = indexes.CharField(null=True)
    last_name = indexes.CharField(null=True)
    email = indexes.CharField(null=True)

    def prepare_mem_type(self, obj):
        type = obj.membership_type.pk
        return type

    def prepare_first_name(self, obj):
        first_name = obj.user.first_name
        return first_name

    def prepare_last_name(self, obj):
        last_name = obj.user.last_name
        return last_name

    def prepare_email(self, obj):
        email = obj.user.email
        return email


class MemberAppIndex(TendenciBaseSearchIndex):
    name = indexes.CharField(model_attr='name')
    description = indexes.CharField(model_attr='description')

    def prepare_description(self, obj):
        description = obj.description
        description = strip_tags(description)
        description = strip_entities(description)
        return description


class MemberAppEntryIndex(TendenciBaseSearchIndex):
    entry_time = indexes.DateTimeField(model_attr='entry_time')

    def get_updated_field(self):
        return 'entry_time'


site.register(Membership, MembershipIndex)
site.register(App, MemberAppIndex)
site.register(AppEntry, MemberAppEntryIndex)
